@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with Atari800; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include "atari.h"
@@ -30,18 +30,20 @@
 #include "sound.h"
 #endif
 
-void Atari_Initialise(int *argc, char *argv[])
+int PLATFORM_Initialise(int *argc, char *argv[])
 {
 #ifdef SOUND
 	Sound_Initialise(argc, argv);
 #endif
+
+	return TRUE;
 }
 
-int Atari_Exit(int run_monitor)
+int PLATFORM_Exit(int run_monitor)
 {
-	Aflushlog();
+	Log_flushlog();
 
-	if (run_monitor && monitor())
+	if (run_monitor && MONITOR_Run())
 		return TRUE;
 
 #ifdef SOUND
