@@ -2,7 +2,7 @@
  * cassette.c - cassette emulation
  *
  * Copyright (C) 2001 Piotr Fusik
- * Copyright (C) 2001-2005 Atari800 development team (see DOC/CREDITS)
+ * Copyright (C) 2001-2008 Atari800 development team (see DOC/CREDITS)
  *
  * This file is part of the Atari800 emulator project which emulates
  * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with Atari800; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include "config.h"
@@ -33,32 +33,36 @@
 #include "memory.h"
 #include "sio.h"
 #include "util.h"
+#include "pokey.h"
 
 #define MAX_BLOCKS 2048
 
 static FILE *cassette_file = NULL;
 static int cassette_isCAS;
-UBYTE cassette_buffer[4096];
+UBYTE CASSETTE_buffer[4096];
 static ULONG cassette_block_offset[MAX_BLOCKS];
 static SLONG cassette_elapsedtime;  /* elapsed time since begin of file */
                                     /* in scanlines */
+static SLONG cassette_savetime;	    /* helper for cas save */
 static SLONG cassette_nextirqevent; /* timestamp of next irq in scanlines */
 
-char cassette_filename[FILENAME_MAX];
-char cassette_description[CASSETTE_DESCRIPTION_MAX];
-int cassette_current_blockbyte = 0;
-int cassette_current_block;
-int cassette_max_blockbytes = 0;
-int cassette_max_block = 0;
-int cassette_savefile = FALSE;
-int cassette_gapdelay = 0;	/* in ms, includes leader and all gaps */
-int cassette_motor = 0;
-int cassette_baudrate = 600;	/* provisional: 600 baud */
+static char cassette_filename[FILENAME_MAX];
+static char cassette_description[CASSETTE_DESCRIPTION_MAX];
+static int cassette_current_blockbyte = 0;
+static int cassette_current_block;
+static int cassette_max_blockbytes = 0;
+static int cassette_max_block = 0;
+static int cassette_savefile = FALSE;
+static int cassette_gapdelay = 0;	/* in ms, includes leader and all gaps */
+static int cassette_putdelay = 0;	/* in ms, delay since last putbyte */
+static int cassette_motor = 0;
+static int cassette_baudrate = 600;
+static int cassette_baudblock[MAX_BLOCKS];
 
-int hold_start_on_reboot = 0;
-int hold_start = 0;
-int press_space = 0;
-int eof_of_tape = 0;
+int CASSETTE_hold_start_on_reboot = 0;
+int CASSETTE_hold_start = 0;
+int CASSETTE_press_space = 0;
+static int eof_of_tape = 0;
 
 typedef struct {
 	char identifier[4];
@@ -84,7 +88,7 @@ $84 as used by the OS), aux is length of mark tone (including leader and gaps)
 just before the record data in milliseconds.
 */
 
-void CASSETTE_Initialise(int *argc, char *argv[])
+int CASSETTE_Initialise(int *argc, char *argv[])
 {
 	int i;
 	int j;
@@ -93,22 +97,38 @@ void CASSETTE_Initialise(int *argc, char *argv[])
 	memset(cassette_description, 0, sizeof(cassette_description));
 
 	for (i = j = 1; i < *argc; i++) {
-		if (strcmp(argv[i], "-tape") == 0)
-			CASSETTE_Insert(argv[++i]);
+		int i_a = (i + 1 < *argc);		/* is argument available? */
+		int a_m = FALSE;			/* error, argument missing! */
+
+		if (strcmp(argv[i], "-tape") == 0) {
+			if (i_a)
+				CASSETTE_Insert(argv[++i]);
+			else a_m = TRUE;
+		}
 		else if (strcmp(argv[i], "-boottape") == 0) {
-			CASSETTE_Insert(argv[++i]);
-			hold_start = 1;
+			if (i_a) {
+				CASSETTE_Insert(argv[++i]);
+				CASSETTE_hold_start = 1;
+			}
+			else a_m = TRUE;
 		}
 		else {
 			if (strcmp(argv[i], "-help") == 0) {
-				Aprint("\t-tape <file>     Insert cassette image");
-				Aprint("\t-boottape <file> Insert cassette image and boot it");
+				Log_print("\t-tape <file>     Insert cassette image");
+				Log_print("\t-boottape <file> Insert cassette image and boot it");
 			}
 			argv[j++] = argv[i];
 		}
+
+		if (a_m) {
+			Log_print("Missing argument for '%s'", argv[i]);
+			return FALSE;
+		}
 	}
 
 	*argc = j;
+
+	return TRUE;
 }
 
 /* Gets information about the cassette image. Returns TRUE if ok.
@@ -125,6 +145,7 @@ int CASSETTE_CheckFile(const char *filename, FILE **fp, char *description, int *
 	int blocks;
 
 /* atm, no appending to existing cas files - too dangerous to data!
+   and will not work anyway without patched sio
 	f = fopen(filename, "rb+");
 	if (f == NULL) */
 		f = fopen(filename, "rb");
@@ -132,6 +153,7 @@ int CASSETTE_CheckFile(const char *filename, FILE **fp, char *description, int *
 		return FALSE;
 	if (description)
 		memset(description, 0, CASSETTE_DESCRIPTION_MAX);
+
 	if (fread(&header, 1, 6, f) == 6
 		&& header.identifier[0] == 'F'
 		&& header.identifier[1] == 'U'
@@ -157,17 +179,26 @@ int CASSETTE_CheckFile(const char *filename, FILE **fp, char *description, int *
 
 		/* count number of blocks */
 		blocks = 0;
+		cassette_baudblock[0]=600;
 		do {
 			/* chunk header is always 8 bytes */
 			if (fread(&header, 1, 8, f) != 8)
 				break;
-			if (header.identifier[0] == 'd'
+			if (header.identifier[0] == 'b'
+			 && header.identifier[1] == 'a'
+			 && header.identifier[2] == 'u'
+			 && header.identifier[3] == 'd') {
+				cassette_baudrate=header.aux_lo + (header.aux_hi << 8);
+			}
+			else if (header.identifier[0] == 'd'
 			 && header.identifier[1] == 'a'
 			 && header.identifier[2] == 't'
 			 && header.identifier[3] == 'a') {
 				blocks++;
-				if (fp)
+				if (fp) {
 					cassette_block_offset[blocks] = ftell(f) - 4;
+					cassette_baudblock[blocks] = cassette_baudrate;
+				}
 			}
 			length = header.length_lo + (header.length_hi << 8);
 			/* skip possibly present data block */
@@ -183,6 +214,11 @@ int CASSETTE_CheckFile(const char *filename, FILE **fp, char *description, int *
 	}
 	if (last_block)
 		*last_block = blocks;
+	/* if filelength is 0, then it is a savefile */
+	fseek(f,0L,SEEK_SET);
+	if (fread(&header, 1, 4, f) == 0) {
+		cassette_savefile = TRUE;
+	}
 	if (fp)
 		*fp = f;
 	else
@@ -244,14 +280,18 @@ int CASSETTE_CreateFile(const char *filename, FILE **fp, int *isCAS)
 
 int CASSETTE_Insert(const char *filename)
 {
+	CASSETTE_Remove();
 	strcpy(cassette_filename, filename);
 	cassette_elapsedtime = 0;
+	cassette_savetime = 0;
 	cassette_nextirqevent = 0;
 	cassette_current_blockbyte = 0;
 	cassette_max_blockbytes = 0;
 	cassette_current_block = 1;
+	cassette_gapdelay = 0;
 	eof_of_tape = 0;
 	cassette_savefile = FALSE;
+	cassette_baudrate = 600;
 	return CASSETTE_CheckFile(filename, &cassette_file, cassette_description,
 		 &cassette_max_block, &cassette_isCAS);
 }
@@ -259,6 +299,10 @@ int CASSETTE_Insert(const char *filename)
 void CASSETTE_Remove(void)
 {
 	if (cassette_file != NULL) {
+		/* save last block, ignore trailing space tone */
+		if ((cassette_current_blockbyte > 0) && cassette_savefile)
+			CASSETTE_Write(cassette_current_blockbyte);
+
 		fclose(cassette_file);
 		cassette_file = NULL;
 	}
@@ -269,9 +313,9 @@ void CASSETTE_Remove(void)
 
 /* Read a record by SIO-patch
    returns block length (with checksum) */
-static UWORD ReadRecord_SIO(void)
+static int ReadRecord_SIO(void)
 {
-	UWORD length = 0;
+	int length = 0;
 	if (cassette_isCAS) {
 		CAS_Header header;
 		/* if waiting for gap was longer than gap of record, skip
@@ -288,37 +332,36 @@ static UWORD ReadRecord_SIO(void)
 			/* add gaplength */
 			filegaptimes += header.aux_lo + (header.aux_hi << 8);
 			/* add time used by the data themselves
-			   atm, assumes a baud rate of 600 (a byte is encoded
-			   into 10 bits -> 60 bytes per second) */
-			filegaptimes += length * 1000 / 60;
+			   a byte is encoded into 10 bits */
+			filegaptimes += length * 10 * 1000 / cassette_baudblock[cassette_current_block];
 
-			fread(&cassette_buffer[0], 1, length, cassette_file);
+			fread(&CASSETTE_buffer[0], 1, length, cassette_file);
 			cassette_current_block++;
 		}
 		cassette_gapdelay = 0;
 	}
 	else {
 		length = 132;
-		cassette_buffer[0] = 0x55;
-		cassette_buffer[1] = 0x55;
+		CASSETTE_buffer[0] = 0x55;
+		CASSETTE_buffer[1] = 0x55;
 		if (cassette_current_block >= cassette_max_block) {
 			/* EOF record */
-			cassette_buffer[2] = 0xfe;
-			memset(cassette_buffer + 3, 0, 128);
+			CASSETTE_buffer[2] = 0xfe;
+			memset(CASSETTE_buffer + 3, 0, 128);
 		}
 		else {
 			int bytes;
 			fseek(cassette_file, (cassette_current_block - 1) * 128, SEEK_SET);
-			bytes = fread(cassette_buffer + 3, 1, 128, cassette_file);
+			bytes = fread(CASSETTE_buffer + 3, 1, 128, cassette_file);
 			if (bytes < 128) {
-				cassette_buffer[2] = 0xfa;	/* non-full record */
-				memset(cassette_buffer + 3 + bytes, 0, 127 - bytes);
-				cassette_buffer[0x82] = bytes;
+				CASSETTE_buffer[2] = 0xfa;	/* non-full record */
+				memset(CASSETTE_buffer + 3 + bytes, 0, 127 - bytes);
+				CASSETTE_buffer[0x82] = bytes;
 			}
 			else
-				cassette_buffer[2] = 0xfc;	/* full record */
+				CASSETTE_buffer[2] = 0xfc;	/* full record */
 		}
-		cassette_buffer[0x83] = SIO_ChkSum(cassette_buffer, 0x83);
+		CASSETTE_buffer[0x83] = SIO_ChkSum(CASSETTE_buffer, 0x83);
 		cassette_current_block++;
 	}
 	return length;
@@ -332,7 +375,7 @@ static UWORD WriteRecord(int len)
 	/* always append */
 	fseek(cassette_file, 0, SEEK_END);
 	/* write recordheader */
-	strncpy(header.identifier, "data", 4);
+	Util_strncpy(header.identifier, "data", 4);
 	header.length_lo = len & 0xFF;
 	header.length_hi = (len >> 8) & 0xFF;
 	header.aux_lo = cassette_gapdelay & 0xff;
@@ -342,7 +385,7 @@ static UWORD WriteRecord(int len)
 	cassette_max_block++;
 	cassette_current_block++;
 	/* write record */
-	return fwrite(&cassette_buffer, 1, len, cassette_file);
+	return fwrite(&CASSETTE_buffer, 1, len, cassette_file);
 }
 
 int CASSETTE_AddGap(int gaptime)
@@ -358,9 +401,9 @@ void CASSETTE_LeaderLoad(void)
 {
 	cassette_gapdelay = 9600;
 	/* registers for SETVBV: third system timer, ~0.1 sec */
-	regA = 3;
-	regX = 0;
-	regY = 5;
+	CPU_regA = 3;
+	CPU_regX = 0;
+	CPU_regY = 5;
 }
 
 /* indicates that a save leader is written by the OS */
@@ -368,9 +411,9 @@ void CASSETTE_LeaderSave(void)
 {
 	cassette_gapdelay = 19200;
 	/* registers for SETVBV: third system timer, ~0.1 sec */
-	regA = 3;
-	regX = 0;
-	regY = 5;
+	CPU_regA = 3;
+	CPU_regX = 0;
+	CPU_regY = 5;
 }
 
 /* Read Cassette Record from Storage medium
@@ -379,7 +422,7 @@ void CASSETTE_LeaderSave(void)
     must be split and a baud chunk inserted inbetween) or -1 for error */
 int CASSETTE_Read(void)
 {
-	/* no file or blockpositions dont match anymore after saving */
+	/* no file or blockpositions don't match anymore after saving */
 	if ((cassette_file == NULL) || (cassette_savefile == TRUE))
 		return -1;
 	if (cassette_current_block > cassette_max_block)
@@ -395,7 +438,7 @@ int CASSETTE_Write(int length)
 	/* there must be a filename given for saving */
 	if (strcmp(cassette_filename, "None") == 0)
 		return -1;
-	/* if file doesnt exist (or has no records), create the header */
+	/* if file doesn't exist (or has no records), create the header */
 	if ((cassette_file == NULL || ftell(cassette_file) == 0) &&
 			(CASSETTE_CreateFile(cassette_filename,
 			&cassette_file,&cassette_isCAS) == FALSE))
@@ -410,7 +453,7 @@ int CASSETTE_Write(int length)
 }
 
 /* convert milliseconds to scanlines */
-SLONG MSToScanLines(int ms)
+static SLONG MSToScanLines(int ms)
 {
 	/* for PAL resolution, deviation in NTSC is negligible */
 	return 312*50*ms/1000;
@@ -419,12 +462,12 @@ SLONG MSToScanLines(int ms)
 /* Support to read a record by POKEY-registers
    evals gap length
    returns block length (with checksum) */
-static UWORD ReadRecord_POKEY(void)
+static int ReadRecord_POKEY(void)
 {
-	UWORD length = 0;
+	int length = 0;
 	if (cassette_isCAS) {
 		CAS_Header header;
-		/* no file or blockpositions dont match anymore after saving */
+		/* no file or blockpositions don't match anymore after saving */
 		if ((cassette_file == NULL) || (cassette_savefile == TRUE)) {
 			cassette_nextirqevent = -1;
 			length = -1;
@@ -446,9 +489,10 @@ static UWORD ReadRecord_POKEY(void)
 			cassette_nextirqevent = cassette_elapsedtime +
 				MSToScanLines(
 				header.aux_lo + (header.aux_hi << 8)
-				+ 10 * 1000 / cassette_baudrate);
+				+ 10 * 1000 / cassette_baudblock[
+				cassette_current_block]);
 			/* read block into buffer */
-			fread(&cassette_buffer[0], 1, length, cassette_file);
+			fread(&CASSETTE_buffer[0], 1, length, cassette_file);
 			cassette_max_blockbytes = length;
 			cassette_current_blockbyte = 0;
 			cassette_current_block++;
@@ -456,12 +500,12 @@ static UWORD ReadRecord_POKEY(void)
 	}
 	else {
 		length = 132;
-		cassette_buffer[0] = 0x55;
-		cassette_buffer[1] = 0x55;
+		CASSETTE_buffer[0] = 0x55;
+		CASSETTE_buffer[1] = 0x55;
 		if (cassette_current_block >= cassette_max_block) {
 			/* EOF record */
-			cassette_buffer[2] = 0xfe;
-			memset(cassette_buffer + 3, 0, 128);
+			CASSETTE_buffer[2] = 0xfe;
+			memset(CASSETTE_buffer + 3, 0, 128);
 			if (cassette_current_block > cassette_max_block) {
 				eof_of_tape = 1;
 			}
@@ -470,29 +514,29 @@ static UWORD ReadRecord_POKEY(void)
 			int bytes;
 			fseek(cassette_file,
 				(cassette_current_block - 1) * 128, SEEK_SET);
-			bytes = fread(cassette_buffer + 3, 1, 128,
+			bytes = fread(CASSETTE_buffer + 3, 1, 128,
 				cassette_file);
 			if (bytes < 128) {
-				cassette_buffer[2] = 0xfa; /* non-full record */
-				memset(cassette_buffer + 3 + bytes, 0,
+				CASSETTE_buffer[2] = 0xfa; /* non-full record */
+				memset(CASSETTE_buffer + 3 + bytes, 0,
 					127 - bytes);
-				cassette_buffer[0x82] = bytes;
+				CASSETTE_buffer[0x82] = bytes;
 			}
 			else
-				cassette_buffer[2] = 0xfc; /* full record */
+				CASSETTE_buffer[2] = 0xfc; /* full record */
 		}
-		cassette_buffer[0x83] = SIO_ChkSum(cassette_buffer, 0x83);
+		CASSETTE_buffer[0x83] = SIO_ChkSum(CASSETTE_buffer, 0x83);
 		/* eval time to first byte coming out of pokey */
 		if (cassette_current_block == 1) {
 			/* on first block, length includes a leader */
 			cassette_nextirqevent = cassette_elapsedtime +
 				MSToScanLines(19200
-				+ 10 * 1000 / cassette_baudrate);
+				+ 10 * 1000 / 600); /* always 600 baud */
 		}
 		else {
 			cassette_nextirqevent = cassette_elapsedtime +
 				MSToScanLines(260
-				+ 10 * 1000 / cassette_baudrate);
+				+ 10 * 1000 / 600); /* always 600 baud */
 		}
 		cassette_max_blockbytes = length;
 		cassette_current_blockbyte = 0;
@@ -503,14 +547,15 @@ static UWORD ReadRecord_POKEY(void)
 
 /* sets the stamp of next irq and loads new record if necessary
    adjust is a value to correction of time of next irq*/
-int SetNextByteTime_POKEY(int adjust)
+static int SetNextByteTime_POKEY(int adjust)
 {
 	int length = 0;
 	cassette_current_blockbyte += 1;
 	/* if there are still bytes in the buffer, take next byte */
 	if (cassette_current_blockbyte < cassette_max_blockbytes) {
 		cassette_nextirqevent = cassette_elapsedtime + adjust
-		+ MSToScanLines(10 * 1000 / cassette_baudrate);
+			+ MSToScanLines(10 * 1000 / (cassette_isCAS?cassette_baudblock[
+			cassette_current_block-1]:600));
 		return 0;
 	}
 
@@ -529,7 +574,7 @@ int SetNextByteTime_POKEY(int adjust)
 int CASSETTE_GetByte(void)
 {
 	/* there are programs which load 2 blocks as one */
-	return cassette_buffer[cassette_current_blockbyte];
+	return CASSETTE_buffer[cassette_current_blockbyte];
 }
 
 /* Check status of I/O-line
@@ -541,7 +586,7 @@ int CASSETTE_IOLineStatus(void)
 {
 	int bit = 0;
 
-	/* if motor off and EOF return always 1 (eviqualent the mark tone) */
+	/* if motor off and EOF return always 1 (equivalent the mark tone) */
 	if ((cassette_motor == 0)
 		|| (eof_of_tape != 0)) {
 		return 1;
@@ -549,9 +594,12 @@ int CASSETTE_IOLineStatus(void)
 
 	/* exam rate; if elapsed time > nextirq - duration of one byte */
 	if (cassette_elapsedtime >
-		(cassette_nextirqevent - 10 * 50 * 312 / cassette_baudrate + 1)) {
+		(cassette_nextirqevent - 10 * 50 * 312 / 
+		(cassette_isCAS?cassette_baudblock[cassette_current_block-1]:
+		600) + 1)) {
 		bit = (cassette_nextirqevent - cassette_elapsedtime)
-			/ (50 * 312 / cassette_baudrate);
+			/ (50 * 312 / (cassette_isCAS?cassette_baudblock[
+			cassette_current_block-1]:600));
 	}
 	else {
 		bit = 0;
@@ -579,7 +627,8 @@ int CASSETTE_GetInputIRQDelay(void)
 	if ((cassette_file == NULL)
 		|| (eof_of_tape != 0)
 		|| (cassette_motor == 0)
-		|| (cassette_nextirqevent < 0)) {
+		|| (cassette_nextirqevent < 0)
+		|| cassette_savefile) {
 		return 0;
 	}
 
@@ -594,7 +643,8 @@ int CASSETTE_GetInputIRQDelay(void)
 		timespan = cassette_nextirqevent - cassette_elapsedtime;
 	}
 	if (timespan < 40) {
-		timespan += ((312 * 50 - 1) / cassette_baudrate) * 10;
+		timespan += ((312 * 50 - 1) / (cassette_isCAS?cassette_baudblock[
+			cassette_current_block-1]:600)) * 10;
 	}
 
 	/* if still negative, return "failed" */
@@ -604,13 +654,79 @@ int CASSETTE_GetInputIRQDelay(void)
 	return timespan;
 }
 
+/* return if save file */
+int CASSETTE_IsSaveFile(void)
+{
+	return cassette_savefile;
+}
+
+/* put a byte into the cas file */
+/* the block is being written at first putbyte of the subsequent block */
+void CASSETTE_PutByte(int byte)
+{
+	/* get time since last byte-put resp. motor on */
+	cassette_putdelay = 1000*(cassette_elapsedtime-cassette_savetime)/312/50;
+	/* subtract one byte-duration from put delay */
+	cassette_putdelay -= 1000 * 10 * (POKEY_AUDF[POKEY_CHAN3] + POKEY_AUDF[POKEY_CHAN4]*0x100)/895000;
+	/* if putdelay > 5 (ms) */
+	if (cassette_putdelay > 05) {
+
+		/* write previous block */
+		if (cassette_current_blockbyte > 0) {
+			CASSETTE_Write(cassette_current_blockbyte);
+			cassette_current_blockbyte = 0;
+			cassette_gapdelay = 0;
+		}
+		/* set new gap-time */
+		cassette_gapdelay += cassette_putdelay;
+	}
+	/* put byte into buffer */
+	CASSETTE_buffer[cassette_current_blockbyte] = byte;
+	cassette_current_blockbyte++;
+	/* set new last byte-put time */
+	cassette_savetime = cassette_elapsedtime;
+}
+
 /* set motor status
  1 - on, 0 - off
  remark: the real evaluation is done in AddScanLine*/
 void CASSETTE_TapeMotor(int onoff)
 {
 	if (cassette_file != NULL) {
-		cassette_motor = onoff;
+		if (cassette_savefile) { /* cas save */
+			if (onoff != cassette_motor) {
+				if (onoff == 0) {
+					/* get time since last byte-put resp. motor on */
+					cassette_putdelay = 1000*(cassette_elapsedtime-cassette_savetime)/312/50;
+					/* subtract one byte-duration from put delay */
+					cassette_putdelay -= 1000*10 * (POKEY_AUDF[POKEY_CHAN3] + POKEY_AUDF[POKEY_CHAN4]*0x100)/895000;
+					/* set putdelay non-negative */
+        				if (cassette_putdelay < 0) {
+						cassette_putdelay = 0;
+					}
+					/* write block from buffer */
+					if (cassette_current_blockbyte > 0) {
+						CASSETTE_Write(cassette_current_blockbyte);
+						cassette_gapdelay = 0;
+						cassette_current_blockbyte = 0;
+					}
+					/* set new gap-time */
+					cassette_gapdelay += cassette_putdelay;
+
+				}
+				else {
+					/* motor on if it was off - restart gap time */
+					cassette_savetime = cassette_elapsedtime;
+				}
+				cassette_motor = onoff;
+			}
+		}
+		else {
+			cassette_motor = onoff;
+			/* restart gap time */
+			if (onoff == 1)
+				cassette_savetime = cassette_elapsedtime;
+		};
 	}
 }
 
@@ -620,14 +736,16 @@ void CASSETTE_AddScanLine(void)
 	/* if motor on and a cassette file is opened, and not eof */
 	/* increment elapsed cassette time */
 	if (cassette_motor && (cassette_file != NULL)
-		&& (eof_of_tape == 0)) {
+		&& ((eof_of_tape == 0) || cassette_savefile)) {
 		cassette_elapsedtime++;
 
+		/* only for loading: set next byte interrupt time */
 		/* valid cassette times are up to 870 baud, giving
 		   a time span of 18 scanlines, so comparing with
 		   -2 leaves a safe timespan for letting get the bit out
 		   of the pokey */
-		if ((cassette_nextirqevent - cassette_elapsedtime) <= -2) {
+		if (((cassette_nextirqevent - cassette_elapsedtime) <= -2)
+			&& !cassette_savefile) {
 			tmp = SetNextByteTime_POKEY(-2);
 		}
 	}
