@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with Atari800; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <stdio.h>
@@ -35,12 +35,14 @@
 #include "cpu.h"
 #include "colours.h"
 #include "input.h"
-#include "ui.h"         /* for ui_is_active */
+#include "akey.h"
+#include "ui.h"         /* for UI_is_active */
 #include "log.h"
 #include "sound.h"
 #include "monitor.h"
 #include "pcjoy.h"
 #include "screen.h"
+#include "platform.h"
 
 #include "dos/vga_gfx.h"
 
@@ -74,10 +76,10 @@ static int first_col = 32; */
 #define first_col 32
 static int ypos_inc = 1;
 static int vga_ptr_inc = 320;
-static int scr_ptr_inc = ATARI_WIDTH;
+static int scr_ptr_inc = Screen_WIDTH;
 
 #ifdef MONITOR_BREAK
-extern UBYTE break_step;  /* used to prevent switching to gfx mode while doing monitor 'step' command*/
+/* extern UBYTE break_step;*/  /* used to prevent switching to gfx mode while doing monitor 'step' command (defined in monitor.h)*/
 #endif
 
 static int video_mode=1;       /*video mode (0-3)*/
@@ -99,7 +101,7 @@ extern int ATKEYPRESSED;
 static int SHIFT_LEFT = FALSE;
 static int SHIFT_RIGHT = FALSE;
 static int alt_key = FALSE;
-extern int alt_function;
+/* extern int UI_alt_function; */ /* defined in ui.h */
 static int norepkey = FALSE;    /* for "no repeat" of some keys !RS! */
 
 static int PC_keyboard = TRUE;
@@ -138,12 +140,12 @@ int joystick0(int *x, int *y);
 /* - moved to vga_asm.s */
 
 /*read analog joystick and set astick & atrig values*/
-void read_joystick(int centre_x, int centre_y)
+static void read_joystick(int centre_x, int centre_y)
 {
         const int threshold = 50;
         int jsx, jsy;
 
-        astick = STICK_CENTRE;
+        astick = INPUT_STICK_CENTRE;
 
         atrig = (joystick0(&jsx, &jsy) < 3) ? 0 : 1;
 
@@ -158,7 +160,7 @@ void read_joystick(int centre_x, int centre_y)
 }
 
 /*find port for given LPT joystick*/
-int test_LPTjoy(int portno,int *port)
+static int test_LPTjoy(int portno,int *port)
 {
         int addr;
 
@@ -173,19 +175,19 @@ int test_LPTjoy(int portno,int *port)
         return TRUE;
 }
 
-void read_LPTjoy(int port, int joyport)
+static void read_LPTjoy(int port, int joyport)
 {
-        int state = STICK_CENTRE, trigger = 1, val;
+        int state = INPUT_STICK_CENTRE, trigger = 1, val;
 
         val = inportb(port + 1);
         if (!(val & (1 << 4)))
-                state &= STICK_FORWARD;
+                state &= INPUT_STICK_FORWARD;
         else if (!(val & (1 << 5)))
-                state &= STICK_BACK;
+                state &= INPUT_STICK_BACK;
         if (!(val & (1 << 6)))
-                state &= STICK_RIGHT;
+                state &= INPUT_STICK_RIGHT;
         else if ((val & (1 << 7)))
-                state &= STICK_LEFT;
+                state &= INPUT_STICK_LEFT;
 
         if (!(val & (1 << 3)))
                 trigger = 0;
@@ -223,8 +225,8 @@ static void update_leds(void)
 /* KEYBOARD HANDLER                                                           */
 /* -------------------------------------------------------------------------- */
 
-unsigned int last_raw_key; /*last key seen by Atari_Keyboard() */
-unsigned int raw_key;   /*key that will be processed int Atari_keyboard() */
+unsigned int last_raw_key; /*last key seen by PLATFORM_Keyboard() */
+unsigned int raw_key;   /*key that will be processed int PLATFORM_keyboard() */
 unsigned int raw_key_r; /*key actually read from PC keyboard*/
 
 _go32_dpmi_seginfo old_key_handler, new_key_handler;
@@ -234,7 +236,7 @@ static int extended_key_follows = FALSE;
 volatile static int key_leave=0;
 static int ki,kstick,ktrig,k_test; /*variables for key_handler*/
 
-void key_handler(void)
+static void key_handler(void)
 {
     asm("cli; pusha");
     raw_key_r = inportb(0x60);
@@ -278,7 +280,7 @@ void key_handler(void)
         k_test=keyforget[raw_key_r];
 #endif
         /*forget scancodes used by keysticks*/
-        if (joy_keyboard && !ui_is_active && keyforget[raw_key_r]) raw_key_r=0;
+        if (joy_keyboard && !UI_is_active && keyforget[raw_key_r]) raw_key_r=0;
 
         switch (raw_key_r) {
                 case 0xaa:  /*ignore 0xe0 0xaa sent by gray arrows*/
@@ -293,7 +295,7 @@ void key_handler(void)
                         break;
 				case 0x38:
 						alt_key = !key_leave;					/*ALT KEY*/
-						if (!alt_key) alt_function = -1;		/* no alt function */
+						if (!alt_key) UI_alt_function = -1;		/* no alt function */
 						break;
                 case 0x1d:
                 case 0x9d:
@@ -310,22 +312,22 @@ void key_handler(void)
                             /* return all keyboard joysticks to default state */
                             if (joytypes[0]>=joy_keyset0 && joytypes[0]<=joy_keyset3)
                             {
-                              stick0=STICK_CENTRE;
+                              stick0=INPUT_STICK_CENTRE;
                               trig0=1;
                             }
                             if (joytypes[1]>=joy_keyset0 && joytypes[1]<=joy_keyset3)
                             {
-                              stick1=STICK_CENTRE;
+                              stick1=INPUT_STICK_CENTRE;
                               trig1=1;
                             }
                             if (joytypes[2]>=joy_keyset0 && joytypes[2]<=joy_keyset3)
                             {
-                              stick2=STICK_CENTRE;
+                              stick2=INPUT_STICK_CENTRE;
                               trig2=1;
                             }
                             if (joytypes[3]>=joy_keyset0 && joytypes[3]<=joy_keyset3)
                             {
-                              stick3=STICK_CENTRE;
+                              stick3=INPUT_STICK_CENTRE;
                               trig3=1;
                             }
                           }
@@ -339,21 +341,21 @@ void key_handler(void)
                         break;
                 case 0x3c:
                         if (!key_leave)
-                          key_consol &= ~CONSOL_OPTION;	/* OPTION key ON */
+                          INPUT_key_consol &= ~INPUT_CONSOL_OPTION;	/* OPTION key ON */
                         else
-                          key_consol |= CONSOL_OPTION;	/* OPTION key OFF */
+                          INPUT_key_consol |= INPUT_CONSOL_OPTION;	/* OPTION key OFF */
                         break;
                 case 0x3d:
                         if (!key_leave)
-                          key_consol &= ~CONSOL_SELECT;	/* SELECT key ON */
+                          INPUT_key_consol &= ~INPUT_CONSOL_SELECT;	/* SELECT key ON */
                         else
-                          key_consol |= CONSOL_SELECT;	/* SELECT key OFF */
+                          INPUT_key_consol |= INPUT_CONSOL_SELECT;	/* SELECT key OFF */
                         break;
                 case 0x3e:
                         if (!key_leave)
-                          key_consol &= ~CONSOL_START;	/* START key ON */
+                          INPUT_key_consol &= ~INPUT_CONSOL_START;	/* START key ON */
                         else
-                          key_consol |= CONSOL_START;	/* START key OFF */
+                          INPUT_key_consol |= INPUT_CONSOL_START;	/* START key OFF */
                         break;
         }
 
@@ -362,7 +364,7 @@ void key_handler(void)
           for (ki=0;ki<4;ki++)
             if (keyset_used[ki])
             {
-              kstick=STICK_CENTRE;
+              kstick=INPUT_STICK_CENTRE;
               if (keypush[keysets[ki][1]] || keypush[keysets[ki][4]] || keypush[keysets[ki][6]])
                 kstick&=0xfb; /*left*/
               if (keypush[keysets[ki][3]] || keypush[keysets[ki][5]] || keypush[keysets[ki][8]])
@@ -387,7 +389,7 @@ void key_handler(void)
             }
         }
 
-        key_shift = SHIFT_LEFT | SHIFT_RIGHT;
+        INPUT_key_shift = SHIFT_LEFT | SHIFT_RIGHT;
         raw_key_r=(key_leave?0x200:0)|raw_key_r;
         if (raw_key_r==0x200) raw_key_r=0;
         if (raw_key==last_raw_key && raw_key_r!=0) {raw_key=raw_key_r;raw_key_r=0;}
@@ -396,7 +398,7 @@ void key_handler(void)
     asm("popa; sti");
 }
 
-void key_init(void)
+static void key_init(void)
 {
 	int i;
 	for (i=0;i<256;i++) keypush[i]=0; /*none key is pressed*/
@@ -411,7 +413,7 @@ void key_init(void)
 	keyboard_handler_replaced = TRUE;
 }
 
-void key_delete(void)
+static void key_delete(void)
 {
         if (keyboard_handler_replaced) {
                 int kflags;
@@ -430,13 +432,32 @@ void key_delete(void)
 /* VGA HANDLING                                                               */
 /* -------------------------------------------------------------------------- */
 
-void SetupVgaEnvironment(void)
-{
+/* Set up the Palette */
+void PLATFORM_PaletteUpdate(void) {
         int a, i;
-        union REGS rg;
         __dpmi_regs d_rg;
         UBYTE ctab[768];
 
+        /* setting all palette at once is faster....*/
+        for (a = 0, i = 0; a < 256; a++) {
+          ctab[i++]=(Colours_table[a] >> 18) & 0x3f;
+          ctab[i++]=(Colours_table[a] >> 10) & 0x3f;
+          ctab[i++]=(Colours_table[a] >> 2) & 0x3f;
+        }
+        /*copy ctab to djgpp transfer buffer in DOS memory*/
+        dosmemput(ctab,768,__tb&0xfffff);
+        d_rg.x.ax=0x1012;
+        d_rg.x.bx=0;
+        d_rg.x.cx=256;
+        d_rg.x.dx=__tb&0xf; /*offset of transfer buffer*/
+        d_rg.x.es=(__tb >> 4) & 0xffff;  /*segment of transfer buffer*/
+        __dpmi_int(0x10,&d_rg);  /*VGA set palette block*/
+}
+
+static void SetupVgaEnvironment(void)
+{
+        union REGS rg;
+
         if (use_vesa)
         {
           /*try to open VESA mode*/
@@ -458,49 +479,36 @@ void SetupVgaEnvironment(void)
               break;
           }
 
-	screen_visible_x1 = first_col;
-	screen_visible_x2 = first_col + 320;
+	Screen_visible_x1 = first_col;
+	Screen_visible_x2 = first_col + 320;
 	if (video_mode == 0) {
-		screen_visible_y1 = first_lno;
-		screen_visible_y2 = first_lno + 200;
+		Screen_visible_y1 = first_lno;
+		Screen_visible_y2 = first_lno + 200;
 	}
 	else {
-		screen_visible_y1 = 0;
-		screen_visible_y2 = 240;
+		Screen_visible_y1 = 0;
+		Screen_visible_y2 = 240;
 	}
 
         vga_started = 1;
 
-        /* setting all palette at once is faster....*/
-        for (a = 0, i = 0; a < 256; a++) {
-          ctab[i++]=(colortable[a] >> 18) & 0x3f;
-          ctab[i++]=(colortable[a] >> 10) & 0x3f;
-          ctab[i++]=(colortable[a] >> 2) & 0x3f;
-        }
-        /*copy ctab to djgpp transfer buffer in DOS memory*/
-        dosmemput(ctab,768,__tb&0xfffff);
-        d_rg.x.ax=0x1012;
-        d_rg.x.bx=0;
-        d_rg.x.cx=256;
-        d_rg.x.dx=__tb&0xf; /*offset of transfer buffer*/
-        d_rg.x.es=(__tb >> 4) & 0xffff;  /*segment of transfer buffer*/
-        __dpmi_int(0x10,&d_rg);  /*VGA set palette block*/
+	PLATFORM_PaletteUpdate();
 
 	/* initialize mouse */
-	if (mouse_mode != MOUSE_OFF) {
+	if (INPUT_mouse_mode != INPUT_MOUSE_OFF) {
 		union REGS rg;
 		rg.x.ax = 0;
 		int86(0x33, &rg, &rg);
 		if (rg.x.ax != 0xffff) {
-			Aprint("Can't find mouse!");
-			mouse_mode = MOUSE_OFF;
+			Log_print("Can't find mouse!");
+			INPUT_mouse_mode = INPUT_MOUSE_OFF;
 		}
 	}
 
         key_init(); /*initialize keyboard handler*/
 }
 
-void ShutdownVgaEnvironment(void)
+static void ShutdownVgaEnvironment(void)
 {
         union REGS rg;
 
@@ -516,9 +524,9 @@ void ShutdownVgaEnvironment(void)
    vga_started=FALSE;
 }
 
-void Atari_DisplayScreen(void)
+void PLATFORM_DisplayScreen(void)
 {
-        UBYTE *ascreen = (UBYTE *) atari_screen;
+        UBYTE *ascreen = (UBYTE *) Screen_atari;
         static int lace = 0;
         unsigned long vga_ptr;
 
@@ -526,12 +534,12 @@ void Atari_DisplayScreen(void)
         int ypos;
 
         vga_ptr = 0xa0000;
-        scr_ptr = &ascreen[first_lno * ATARI_WIDTH + first_col];
+        scr_ptr = &ascreen[first_lno * Screen_WIDTH + first_col];
 
         if (ypos_inc == 2) {
                 if (lace) {
                         vga_ptr += 320;
-                        scr_ptr += ATARI_WIDTH;
+                        scr_ptr += Screen_WIDTH;
                 }
                 lace = 1 - lace;
         }
@@ -547,20 +555,20 @@ void Atari_DisplayScreen(void)
             switch (video_mode)
             {
               case 0:
-                VESA_blit(ascreen+first_col + first_lno*ATARI_WIDTH,
-                          320,200,ATARI_WIDTH,vesa_linelenght,vesa_selector);
+                VESA_blit(ascreen+first_col + first_lno*Screen_WIDTH,
+                          320,200,Screen_WIDTH,vesa_linelenght,vesa_selector);
                 break;
               case 1:
                 VESA_blit(ascreen+first_col,
-                          320,240,ATARI_WIDTH,vesa_linelenght,vesa_selector);
+                          320,240,Screen_WIDTH,vesa_linelenght,vesa_selector);
                 break;
               case 2:
                 VESA_blit(ascreen+first_col,
-                          320,240,ATARI_WIDTH,vesa_linelenght*2,vesa_selector);
+                          320,240,Screen_WIDTH,vesa_linelenght*2,vesa_selector);
                 break;
               case 3:
                 VESA_i_blit(ascreen+first_col,
-                          320,240,ATARI_WIDTH,vesa_linelenght,vesa_selector);
+                          320,240,Screen_WIDTH,vesa_linelenght,vesa_selector);
                 break;
             }
         else
@@ -576,13 +584,13 @@ void Atari_DisplayScreen(void)
                 }
                 break;
               case 1:
-                x_blit(ascreen+first_col,240,ATARI_WIDTH,80);
+                x_blit(ascreen+first_col,240,Screen_WIDTH,80);
                 break;
               case 2:
-                x_blit(ascreen+first_col,240,ATARI_WIDTH,160);
+                x_blit(ascreen+first_col,240,Screen_WIDTH,160);
                 break;
               case 3:
-                x_blit_i2(ascreen+first_col,240,ATARI_WIDTH,160);
+                x_blit_i2(ascreen+first_col,240,Screen_WIDTH,160);
                 break;
             }
       } /* if (vga_started) */
@@ -592,7 +600,7 @@ void Atari_DisplayScreen(void)
 /* CONFIG & INITIALISATION                                                    */
 /* -------------------------------------------------------------------------- */
 
-void Atari_Initialise(int *argc, char *argv[])
+int PLATFORM_Initialise(int *argc, char *argv[])
 {
         int i;
         int j;
@@ -600,10 +608,13 @@ void Atari_Initialise(int *argc, char *argv[])
         int help_only = FALSE;
 
         for (i = j = 1; i < *argc; i++) {
+		int i_a = (i + 1 < *argc);		/* is argument available? */
+		int a_m = FALSE;			/* error, argument missing! */
+		
                 if (strcmp(argv[i], "-interlace") == 0) {
                         ypos_inc = 2;
                         vga_ptr_inc = 320 + 320;
-                        scr_ptr_inc = ATARI_WIDTH + ATARI_WIDTH;
+                        scr_ptr_inc = Screen_WIDTH + Screen_WIDTH;
                 }
                 else if (strcmp(argv[i], "-LPTjoy1") == 0)
                         use_lpt1=test_LPTjoy(1,NULL);
@@ -615,15 +626,16 @@ void Atari_Initialise(int *argc, char *argv[])
                 else if (strcmp(argv[i], "-joyswap") == 0)
                         joyswap = TRUE;
 
-                else if (strcmp(argv[i], "-video") == 0)
-                {
-                  i++;
-                  video_mode=atoi(argv[i]);
-                  if (video_mode<0 || video_mode>3)
-                  {
-                    Aprint("Invalid video mode, using default.");
-                    video_mode=0;
-                  }
+                else if (strcmp(argv[i], "-video") == 0) {
+			if (i_a) {
+				i++;
+				video_mode=atoi(argv[i]);
+				if (video_mode<0 || video_mode>3) {
+					Log_print("Invalid video mode - should be between 0 and 3");
+					return FALSE;
+				}
+			}
+			else a_m = TRUE;
                 }
                 else if (strcmp(argv[i],"-novesa") == 0)
                 {
@@ -633,33 +645,40 @@ void Atari_Initialise(int *argc, char *argv[])
                 {
                   use_vret=TRUE;
                 }
-		else if (strcmp(argv[i],"-keyboard") == 0)
-		{
-			i++;
-			if (strcmp(argv[i],"0") == 0)
-				PC_keyboard = TRUE;
-			else
-				PC_keyboard = FALSE;
+		else if (strcmp(argv[i],"-keyboard") == 0) {
+			if (i_a) {
+				i++;
+				if (strcmp(argv[i],"0") == 0)
+					PC_keyboard = TRUE;
+				else
+					PC_keyboard = FALSE;
+			}
+			else a_m = TRUE;
 		}
                 else {
                         if (strcmp(argv[i], "-help") == 0) {
                                 help_only = TRUE;
-                                Aprint("\t-interlace       Generate screen with interlace");
-                                Aprint("\t-LPTjoy1         Read joystick connected to LPT1");
-                                Aprint("\t-LPTjoy2         Read joystick connected to LPT2");
-                                Aprint("\t-LPTjoy3         Read joystick connected to LPT3");
-                                Aprint("\t-joyswap         Swap joysticks");
-                                Aprint("\t-video x         Set video mode:");
-                                Aprint("\t\t0 - 320x200\n\t\t1 - 320x240");
-                                Aprint("\t\t2 - 320x240, interlaced with black lines");
-                                Aprint("\t\t3 - 320x240, interlaced with darker lines (slower!)");
-                                Aprint("\t-novesa          Do not use vesa2 videomodes");
-                                Aprint("\t-vretrace        Use vertical retrace control");
-                                Aprint("\t-keyboard 0      PC keyboard layout");
-                                Aprint("\t-keyboard 1      Atari keyboard layout");
+                                Log_print("\t-interlace       Generate screen with interlace");
+                                Log_print("\t-LPTjoy1         Read joystick connected to LPT1");
+                                Log_print("\t-LPTjoy2         Read joystick connected to LPT2");
+                                Log_print("\t-LPTjoy3         Read joystick connected to LPT3");
+                                Log_print("\t-joyswap         Swap joysticks");
+                                Log_print("\t-video x         Set video mode:");
+                                Log_print("\t\t0 - 320x200\n\t\t1 - 320x240");
+                                Log_print("\t\t2 - 320x240, interlaced with black lines");
+                                Log_print("\t\t3 - 320x240, interlaced with darker lines (slower!)");
+                                Log_print("\t-novesa          Do not use vesa2 videomodes");
+                                Log_print("\t-vretrace        Use vertical retrace control");
+                                Log_print("\t-keyboard 0      PC keyboard layout");
+                                Log_print("\t-keyboard 1      Atari keyboard layout");
                         }
                         argv[j++] = argv[i];
                 }
+
+		if (a_m) {
+			Log_print("Missing argument for '%s'", argv[i]);
+			return FALSE;
+		}
         }
 
         *argc = j;
@@ -670,7 +689,7 @@ void Atari_Initialise(int *argc, char *argv[])
 #endif
 
         if (help_only)
-                return;
+                return TRUE;
 
         /* check if joystick is connected */
         printf("Joystick is checked...\n");
@@ -681,7 +700,7 @@ void Atari_Initialise(int *argc, char *argv[])
         if (joy_in)
                 joystick0(&js0_centre_x, &js0_centre_y);
         if (! joy_in)
-                Aprint("Sorry, I see no joystick. Use numeric pad");
+                Log_print("Sorry, I see no joystick. Use numeric pad");
 
         /*find number of VESA2 video mode*/
         if (use_vesa)
@@ -702,7 +721,7 @@ void Atari_Initialise(int *argc, char *argv[])
         }
 
         /* setup joystick */
-        stick0 = stick1 = stick2 = stick3 = STICK_CENTRE;
+        stick0 = stick1 = stick2 = stick3 = INPUT_STICK_CENTRE;
         trig0 = trig1 = trig2 = trig3 = 1;
         /* for compatibility with older versions' command line parameters */
         if (use_lpt3)
@@ -768,11 +787,12 @@ void Atari_Initialise(int *argc, char *argv[])
 
         SetupVgaEnvironment();
 
+	return TRUE;
 }
 
 /* -------------------------------------------------------------------------- */
-/* Atari_Configure(...) processes all lines not recognized by RtConfigLoad() */
-int Atari_Configure(char* option,char* parameters)
+/* PLATFORM_Configure(...) processes all lines not recognized by RtConfigLoad() */
+int PLATFORM_Configure(char* option,char* parameters)
 {
   int help=0;
   int no;
@@ -847,22 +867,22 @@ int Atari_Configure(char* option,char* parameters)
 /* ATARI EXIT                                                                 */
 /* -------------------------------------------------------------------------- */
 
-int Atari_Exit(int run_monitor)
+int PLATFORM_Exit(int run_monitor)
 {
         /* restore to text mode */
         ShutdownVgaEnvironment();
 
         key_delete();                           /* enable keyboard in monitor */
 
-        Aflushlog();
+        Log_flushlog();
 
         if (run_monitor) {
 #ifdef SOUND
                 Sound_Pause();
 #endif
-                if (monitor()) {
+                if (MONITOR_Run()) {
 #ifdef MONITOR_BREAK
-                        if (!break_step)       /*do not enter videomode when stepping through the code*/
+                        if (!MONITOR_break_step)       /*do not enter videomode when stepping through the code*/
 #endif
                         SetupVgaEnvironment();
 #ifdef SOUND
@@ -891,7 +911,7 @@ int Atari_Exit(int run_monitor)
    raw_key & 0x200 = 1 if key was depressed
                                                 Robert Golias
 */
-int Atari_Keyboard(void)
+int PLATFORM_Keyboard(void)
 {
         int i;
         int keycode;
@@ -924,7 +944,7 @@ int Atari_Keyboard(void)
  * This needs a bit of tidying up - array lookup
  */
 /* Atari5200 stuff */
-        if (machine_type == MACHINE_5200 && !ui_is_active) {
+        if (Atari800_machine_type == Atari800_MACHINE_5200 && !UI_is_active) {
                 switch (raw_key) {
                 case 0x3b:
                         keycode = AKEY_UI;
@@ -935,7 +955,7 @@ int Atari_Keyboard(void)
                 case 0x44:                              /* F10 */
                         if (!norepkey)
                         {
-                            keycode = key_shift ? AKEY_SCREENSHOT_INTERLACE : AKEY_SCREENSHOT;
+                            keycode = INPUT_key_shift ? AKEY_SCREENSHOT_INTERLACE : AKEY_SCREENSHOT;
                             norepkey=TRUE;
                         }else
                             keycode = AKEY_NONE;
@@ -990,7 +1010,7 @@ int Atari_Keyboard(void)
                         break;
                 case 0x42:                              /* F8 */
                         if (!norepkey) {
-                                keycode = Atari_Exit(1) ? AKEY_NONE : AKEY_EXIT;        /* invoke monitor */
+                                keycode = PLATFORM_Exit(1) ? AKEY_NONE : AKEY_EXIT;        /* invoke monitor */
                                 norepkey = TRUE;
                         }
                         else
@@ -1006,7 +1026,7 @@ int Atari_Keyboard(void)
         }
 
         /* preinitialize of keycode */
-        shift_mask = key_shift ? 0x40 : 0;
+        shift_mask = INPUT_key_shift ? 0x40 : 0;
         keycode = shift_mask | (control ? 0x80 : 0);
 
         switch (raw_key) {
@@ -1016,7 +1036,7 @@ int Atari_Keyboard(void)
                         keycode = AKEY_NONE;
 			update_leds();
                 }
-                else if (key_shift) {
+                else if (INPUT_key_shift) {
                         PC_keyboard = FALSE;    /* Atari keyboard mode */
                         keycode = AKEY_NONE;
 			update_leds();
@@ -1025,26 +1045,27 @@ int Atari_Keyboard(void)
                         keycode = AKEY_UI;
                 break;
         case 0xCF:                                      /* gray END*/
+        case 0xC5:                                      /* Pause/Break */
                 keycode = AKEY_BREAK;
                 break;
         case 0x44:                                      /* F10 */
                 if (!norepkey)
                 {
-                     keycode = key_shift ? AKEY_SCREENSHOT_INTERLACE : AKEY_SCREENSHOT;
+                     keycode = INPUT_key_shift ? AKEY_SCREENSHOT_INTERLACE : AKEY_SCREENSHOT;
                      norepkey = TRUE;
                 }
                 else
                      keycode = AKEY_NONE;
                 break;
         case 0x3f:                                      /* F5 */
-                keycode = key_shift ? AKEY_COLDSTART : AKEY_WARMSTART;
+                keycode = INPUT_key_shift ? AKEY_COLDSTART : AKEY_WARMSTART;
                 break;
         case 0x40:                                      /* F6 */
                 keycode = AKEY_HELP;
                 break;
         case 0x42:                                      /* F8 */
-                if (!norepkey && !ui_is_active) {
-                        keycode = Atari_Exit(1) ? AKEY_NONE : AKEY_EXIT;        /* invoke monitor */
+                if (!norepkey && !UI_is_active) {
+                        keycode = PLATFORM_Exit(1) ? AKEY_NONE : AKEY_EXIT;        /* invoke monitor */
                         norepkey = TRUE;
                 }
                 else
@@ -1056,8 +1077,8 @@ int Atari_Keyboard(void)
 		case 0x57:					/* F11 */
 		 		keycode = AKEY_NONE;
 				for (i = 0; i < 4; i++) {
-					if (++joy_autofire[i] > 2)
-						joy_autofire[i] = 0;
+					if (++INPUT_joy_autofire[i] > 2)
+						INPUT_joy_autofire[i] = 0;
 		 		}
 		 		raw_key = 0;	/* avoid continuous change */
 		 		break;
@@ -1067,7 +1088,7 @@ int Atari_Keyboard(void)
         case 0xD0:
         case 0x50:
                 if (PC_keyboard)
-                        keycode = AKEY_DOWN;
+                        keycode ^= (INPUT_key_shift ? AKEY_EQUAL : AKEY_DOWN);
                 else {
                         keycode |= AKEY_EQUAL;
                         keycode ^= shift_mask;
@@ -1076,7 +1097,7 @@ int Atari_Keyboard(void)
         case 0xCB:
         case 0x4b:
                 if (PC_keyboard)
-                        keycode = AKEY_LEFT;
+                        keycode ^= (INPUT_key_shift ? AKEY_PLUS : AKEY_LEFT);
                 else {
                         keycode |= AKEY_PLUS;
                         keycode ^= shift_mask;
@@ -1085,7 +1106,7 @@ int Atari_Keyboard(void)
         case 0xCD:
         case 0x4d:
                 if (PC_keyboard)
-                        keycode = AKEY_RIGHT;
+                        keycode ^= (INPUT_key_shift ? AKEY_ASTERISK : AKEY_RIGHT);
                 else {
                         keycode |= AKEY_ASTERISK;
                         keycode ^= shift_mask;
@@ -1094,7 +1115,7 @@ int Atari_Keyboard(void)
         case 0xC8:
         case 0x48:
                 if (PC_keyboard)
-                        keycode = AKEY_UP;
+                        keycode ^= (INPUT_key_shift ? AKEY_MINUS : AKEY_UP);
                 else {
                         keycode |= AKEY_MINUS;
                         keycode ^= shift_mask;
@@ -1112,7 +1133,7 @@ int Atari_Keyboard(void)
         case 0x03:
                 if (!PC_keyboard)
                         keycode |= AKEY_2;
-                else if (key_shift)
+                else if (INPUT_key_shift && !control)
                         keycode = AKEY_AT;
                 else
                         keycode |= AKEY_2;      /* 2,@ */
@@ -1129,7 +1150,7 @@ int Atari_Keyboard(void)
         case 0x07:
                 if (!PC_keyboard)
                         keycode |= AKEY_6;
-                else if (key_shift)
+                else if (INPUT_key_shift && !control)
                         keycode = AKEY_CIRCUMFLEX;      /* 6,^ */
                 else
                         keycode |= AKEY_6;
@@ -1137,7 +1158,7 @@ int Atari_Keyboard(void)
         case 0x08:
                 if (!PC_keyboard)
                         keycode |= AKEY_7;
-                else if (key_shift)
+                else if (INPUT_key_shift && !control)
                         keycode = AKEY_AMPERSAND;       /* 7,& */
                 else
                         keycode |= AKEY_7;
@@ -1145,7 +1166,7 @@ int Atari_Keyboard(void)
         case 0x09:
                 if (!PC_keyboard)
                         keycode |= AKEY_8;
-                else if (key_shift)
+                else if (INPUT_key_shift && !control)
                         keycode = AKEY_ASTERISK;        /* 8,* */
                 else
                         keycode |= AKEY_8;
@@ -1153,7 +1174,7 @@ int Atari_Keyboard(void)
         case 0x0a:
                 if (!PC_keyboard)
                         keycode |= AKEY_9;
-                else if (key_shift)
+                else if (INPUT_key_shift && !control)
                         keycode = AKEY_PARENLEFT;
                 else
                         keycode |= AKEY_9;      /* 9,( */
@@ -1161,7 +1182,7 @@ int Atari_Keyboard(void)
         case 0x0b:
                 if (!PC_keyboard)
                         keycode |= AKEY_0;
-                else if (key_shift)
+                else if (INPUT_key_shift && !control)
                         keycode = AKEY_PARENRIGHT;      /* 0,) */
                 else
                         keycode |= AKEY_0;
@@ -1169,7 +1190,7 @@ int Atari_Keyboard(void)
         case 0x0c:
                 if (!PC_keyboard)
                         keycode |= AKEY_LESS;
-                else if (key_shift)
+                else if (INPUT_key_shift)
                         keycode = AKEY_UNDERSCORE;
                 else
                         keycode |= AKEY_MINUS;
@@ -1177,7 +1198,7 @@ int Atari_Keyboard(void)
         case 0x0d:
                 if (!PC_keyboard)
                         keycode |= AKEY_GREATER;
-                else if (key_shift)
+                else if (INPUT_key_shift)
                         keycode = AKEY_PLUS;
                 else
                         keycode |= AKEY_EQUAL;
@@ -1197,7 +1218,7 @@ int Atari_Keyboard(void)
 				if (alt_key)
 				{
 					keycode = AKEY_UI;
-					alt_function = MENU_SOUND_RECORDING;	/* ALT+W .. Select system */
+					UI_alt_function = UI_MENU_SOUND_RECORDING;	/* ALT+W .. Select system */
 				}
 				else
                 	keycode |= AKEY_w;
@@ -1209,7 +1230,7 @@ int Atari_Keyboard(void)
 				if (alt_key)
 				{
 					keycode = AKEY_UI;
-					alt_function = MENU_RUN;			/* ALT+R .. Run file */
+					UI_alt_function = UI_MENU_RUN;			/* ALT+R .. Run file */
 				}
 				else
 	                keycode |= AKEY_r;
@@ -1221,7 +1242,7 @@ int Atari_Keyboard(void)
 				if (alt_key)
 				{
 					keycode = AKEY_UI;
-					alt_function = MENU_SYSTEM;			/* ALT+Y .. Select system */
+					UI_alt_function = UI_MENU_SYSTEM;			/* ALT+Y .. Select system */
 				}
 				else
 	                keycode |= AKEY_y;
@@ -1236,7 +1257,7 @@ int Atari_Keyboard(void)
 				if (alt_key)
 				{
 					keycode = AKEY_UI;
-					alt_function = MENU_SOUND;			/* ALT+O .. mono/stereo sound */
+					UI_alt_function = UI_MENU_SOUND;			/* ALT+O .. mono/stereo sound */
 				}
 				else
 	                keycode |= AKEY_o;
@@ -1247,7 +1268,7 @@ int Atari_Keyboard(void)
         case 0x1a:
                 if (!PC_keyboard)
                         keycode |= AKEY_MINUS;
-                else if (control | key_shift)
+                else if (control | INPUT_key_shift)
                         keycode |= AKEY_UP;
                 else
                         keycode = AKEY_BRACKETLEFT;
@@ -1255,7 +1276,7 @@ int Atari_Keyboard(void)
         case 0x1b:
                 if (!PC_keyboard)
                         keycode |= AKEY_EQUAL;
-                else if (control | key_shift)
+                else if (control | INPUT_key_shift)
                         keycode |= AKEY_DOWN;
                 else
                         keycode = AKEY_BRACKETRIGHT;
@@ -1269,7 +1290,7 @@ int Atari_Keyboard(void)
 				if (alt_key)
 				{
 					keycode = AKEY_UI;
-					alt_function = MENU_ABOUT;			/* ALT+A .. About */
+					UI_alt_function = UI_MENU_ABOUT;			/* ALT+A .. About */
 				}
 				else
 					keycode |= AKEY_a;
@@ -1278,7 +1299,7 @@ int Atari_Keyboard(void)
 				if (alt_key)
 				{
 					keycode = AKEY_UI;
-					alt_function = MENU_SAVESTATE;			/* ALT+S .. Save state */
+					UI_alt_function = UI_MENU_SAVESTATE;			/* ALT+S .. Save state */
 				}
 				else
 	                keycode |= AKEY_s;
@@ -1287,7 +1308,7 @@ int Atari_Keyboard(void)
 				if (alt_key)
 				{
 					keycode = AKEY_UI;
-					alt_function = MENU_DISK;			/* ALT+D .. Disk management */
+					UI_alt_function = UI_MENU_DISK;			/* ALT+D .. Disk management */
 				}
 				else
 	                keycode |= AKEY_d;
@@ -1311,7 +1332,7 @@ int Atari_Keyboard(void)
 				if (alt_key)
 				{
 					keycode = AKEY_UI;
-					alt_function = MENU_LOADSTATE;			/* ALT+L .. Load state */
+					UI_alt_function = UI_MENU_LOADSTATE;			/* ALT+L .. Load state */
 				}
 				else
 	                keycode |= AKEY_l;
@@ -1322,7 +1343,7 @@ int Atari_Keyboard(void)
         case 0x28:
                 if (!PC_keyboard)
                         keycode |= AKEY_PLUS;
-                else if (key_shift)
+                else if (INPUT_key_shift)
                         keycode = AKEY_DBLQUOTE;
                 else
                         keycode = AKEY_QUOTE;
@@ -1331,10 +1352,14 @@ int Atari_Keyboard(void)
         case 0x56:                                      /* PC key "\,|" */
                 if (!PC_keyboard)
                         keycode |= AKEY_ASTERISK;
-                else if (key_shift)
+                else if (INPUT_key_shift && !control)
                         keycode = AKEY_BAR;
-                else
-                        keycode |= AKEY_BACKSLASH;
+                else if (alt_key)
+				{
+					keycode = AKEY_PBI_BB_MENU; /* Black Box */
+				}
+				else
+                        keycode |= (control ? AKEY_ESCAPE : AKEY_BACKSLASH);
                 break;
 
 
@@ -1348,7 +1373,7 @@ int Atari_Keyboard(void)
 				if (alt_key)
 				{
 					keycode = AKEY_UI;
-					alt_function = MENU_CARTRIDGE;			/* ALT+C .. Cartridge management */
+					UI_alt_function = UI_MENU_CARTRIDGE;			/* ALT+C .. Cartridge management */
 				}
 				else
 	                keycode |= AKEY_c;
@@ -1368,7 +1393,7 @@ int Atari_Keyboard(void)
         case 0x33:
                 if (!PC_keyboard)
                         keycode |= AKEY_COMMA;
-                else if (key_shift && !control)
+                else if (INPUT_key_shift && !control)
                         keycode = AKEY_LESS;
                 else
                         keycode |= AKEY_COMMA;
@@ -1376,7 +1401,7 @@ int Atari_Keyboard(void)
         case 0x34:
                 if (!PC_keyboard)
                         keycode |= AKEY_FULLSTOP;
-                else if (key_shift && !control)
+                else if (INPUT_key_shift && !control)
                         keycode = AKEY_GREATER;
                 else
                         keycode |= AKEY_FULLSTOP;
@@ -1390,17 +1415,17 @@ int Atari_Keyboard(void)
 
 
         case 0xc7:                                      /* HOME key */
-                keycode |= AKEY_CLEAR;
+                keycode |= (control ? AKEY_LESS : AKEY_CLEAR);
                 break;
         case 0xd2:                                      /* INSERT key */
-                if (key_shift)
-                        keycode = AKEY_INSERT_LINE;
+                if (INPUT_key_shift)
+                        keycode |= AKEY_INSERT_LINE;
                 else
                         keycode = AKEY_INSERT_CHAR;
                 break;
         case 0xd3:                                      /* DELETE key */
-                if (key_shift)
-                        keycode = AKEY_DELETE_LINE;
+                if (INPUT_key_shift)
+                        keycode |= AKEY_DELETE_LINE;
                 else
                         keycode = AKEY_DELETE_CHAR;
                 break;
@@ -1417,7 +1442,7 @@ int Atari_Keyboard(void)
 
 /* -------------------------------------------------------------------------- */
 
-int Atari_PORT(int num)
+int PLATFORM_PORT(int num)
 {
 	if (num == 0)
 		return (stick1 << 4) | stick0;
@@ -1427,7 +1452,7 @@ int Atari_PORT(int num)
 
 /* -------------------------------------------------------------------------- */
 
-int Atari_TRIG(int num)
+int PLATFORM_TRIG(int num)
 {
         switch (num) {
         case 0:
@@ -1452,21 +1477,21 @@ int main(int argc, char **argv)
 
 	/* main loop */
 	for (;;) {
-		key_code = Atari_Keyboard();
+		INPUT_key_code = PLATFORM_Keyboard();
 
-		if (mouse_mode != MOUSE_OFF) {
+		if (INPUT_mouse_mode != INPUT_MOUSE_OFF) {
 			union REGS rg;
 			rg.x.ax = 0x0b;
 			int86(0x33, &rg, &rg);
-			mouse_delta_x = (short) rg.x.cx;	/* signed! */
-			mouse_delta_y = (short) rg.x.dx;	/* signed! */
+			INPUT_mouse_delta_x = (short) rg.x.cx;	/* signed! */
+			INPUT_mouse_delta_y = (short) rg.x.dx;	/* signed! */
 			rg.x.ax = 0x03;
 			int86(0x33, &rg, &rg);
-			mouse_buttons = rg.x.bx;
+			INPUT_mouse_buttons = rg.x.bx;
 		}
 
 		Atari800_Frame();
-		if (display_screen)
-			Atari_DisplayScreen();
+		if (Atari800_display_screen)
+			PLATFORM_DisplayScreen();
 	}
 }
