@@ -1,12 +1,13 @@
-#ifndef _MZPOKEYSND_H_
-#define _MZPOKEYSND_H_
+#ifndef MZPOKEYSND_H_
+#define MZPOKEYSND_H_
 
 #include "config.h"
 #include "pokeysnd.h"
+#include "atari.h"
 
-int Pokey_sound_init_mz(uint32 freq17,
-                        uint16 playback_freq,
-                        uint8 num_pokeys,
+int MZPOKEYSND_Init(ULONG freq17,
+                        int playback_freq,
+                        UBYTE num_pokeys,
                         int flags,
                         int quality
 #ifdef __PLUS
@@ -14,4 +15,4 @@ int Pokey_sound_init_mz(uint32 freq17,
 #endif
                        );
 
-#endif /* _MZPOKEYSND_H_ */
+#endif /* MZPOKEYSND_H_ */
