@@ -1,3 +1,5 @@
+#ifndef RTIME_H_
+#define RTIME_H_
 /* Emulate ICD R-Time 8 cartridge
    Copyright 2000 Jason Duerstock <jason@cluephone.com>
 
@@ -13,12 +15,13 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include "atari.h"
 
-extern int rtime_enabled;
+extern int RTIME_enabled;
 
-void RTIME_Initialise(int *argc, char *argv[]);
+int RTIME_Initialise(int *argc, char *argv[]);
 UBYTE RTIME_GetByte(void);
 void RTIME_PutByte(UBYTE byte);
+#endif /* RTIME_H_ */
