@@ -2,7 +2,7 @@
  * screen.c - Atari screen handling
  *
  * Copyright (c) 2001 Robert Golias and Piotr Fusik
- * Copyright (C) 2001-2005 Atari800 development team (see DOC/CREDITS)
+ * Copyright (C) 2001-2008 Atari800 development team (see DOC/CREDITS)
  *
  * This file is part of the Atari800 emulator project which emulates
  * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with Atari800; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include "config.h"
@@ -42,31 +42,31 @@
 #define ATARI_VISIBLE_WIDTH 336
 #define ATARI_LEFT_MARGIN 24
 
-ULONG *atari_screen = NULL;
+ULONG *Screen_atari = NULL;
 #ifdef DIRTYRECT
-UBYTE *screen_dirty = NULL;
+UBYTE *Screen_dirty = NULL;
 #endif
 #ifdef BITPL_SCR
-ULONG *atari_screen_b = NULL;
-ULONG *atari_screen1 = NULL;
-ULONG *atari_screen2 = NULL;
+ULONG *Screen_atari_b = NULL;
+ULONG *Screen_atari1 = NULL;
+ULONG *Screen_atari2 = NULL;
 #endif
 
-/* The area that can been seen is screen_visible_x1 <= x < screen_visible_x2,
-   screen_visible_y1 <= y < screen_visible_y2.
-   Full Atari screen is 336x240. ATARI_WIDTH is 384 only because
+/* The area that can been seen is Screen_visible_x1 <= x < Screen_visible_x2,
+   Screen_visible_y1 <= y < Screen_visible_y2.
+   Full Atari screen is 336x240. Screen_WIDTH is 384 only because
    the code in antic.c sometimes draws more than 336 bytes in a line.
-   Currently screen_visible variables are used only to place
+   Currently Screen_visible variables are used only to place
    disk led and snailmeter in the corners of the screen.
 */
-int screen_visible_x1 = 24;				/* 0 .. ATARI_WIDTH */
-int screen_visible_y1 = 0;				/* 0 .. ATARI_HEIGHT */
-int screen_visible_x2 = 360;			/* 0 .. ATARI_WIDTH */
-int screen_visible_y2 = ATARI_HEIGHT;	/* 0 .. ATARI_HEIGHT */
+int Screen_visible_x1 = 24;				/* 0 .. Screen_WIDTH */
+int Screen_visible_y1 = 0;				/* 0 .. Screen_HEIGHT */
+int Screen_visible_x2 = 360;			/* 0 .. Screen_WIDTH */
+int Screen_visible_y2 = Screen_HEIGHT;	/* 0 .. Screen_HEIGHT */
 
-int show_atari_speed = FALSE;
-int show_disk_led = TRUE;
-int show_sector_counter = FALSE;
+int Screen_show_atari_speed = FALSE;
+int Screen_show_disk_led = TRUE;
+int Screen_show_sector_counter = FALSE;
 
 #ifdef HAVE_LIBPNG
 #define DEFAULT_SCREENSHOT_FILENAME_FORMAT "atari%03d.png"
@@ -110,47 +110,63 @@ static void Screen_SetScreenshotFilenamePattern(const char *p)
 			return; /* ok */
 		p++;
 	}
-	Aprint("Invalid filename pattern for screenshots, using default.");
+	Log_print("Invalid filename pattern for screenshots, using default.");
 	strcpy(screenshot_filename_format, DEFAULT_SCREENSHOT_FILENAME_FORMAT);
 	screenshot_no_max = 1000;
 }
 
-void Screen_Initialise(int *argc, char *argv[])
+int Screen_Initialise(int *argc, char *argv[])
 {
 	int i;
 	int j;
 	int help_only = FALSE;
 
 	for (i = j = 1; i < *argc; i++) {
+		int i_a = (i + 1 < *argc);		/* is argument available? */
+		int a_m = FALSE;			/* error, argument missing! */
+		
 		if (strcmp(argv[i], "-screenshots") == 0) {
-			Screen_SetScreenshotFilenamePattern(argv[++i]);
+			if (i_a)
+				Screen_SetScreenshotFilenamePattern(argv[++i]);
+			else a_m = TRUE;
+		}
+		if (strcmp(argv[i], "-showspeed") == 0) {
+			Screen_show_atari_speed = TRUE;
 		}
 		else {
 			if (strcmp(argv[i], "-help") == 0) {
 				help_only = TRUE;
-				Aprint("\t-screenshots <p> Set filename pattern for screenshots");
+				Log_print("\t-screenshots <p> Set filename pattern for screenshots");
+				Log_print("\t-showspeed       Show percentage of actual speed");
 			}
 			argv[j++] = argv[i];
 		}
+
+		if (a_m) {
+			Log_print("Missing argument for '%s'", argv[i]);
+			return FALSE;
+		}
 	}
 	*argc = j;
 
-	/* don't bother mallocing atari_screen with just "-help" */
+	/* don't bother mallocing Screen_atari with just "-help" */
 	if (help_only)
-		return;
+		return TRUE;
 
-	if (atari_screen == NULL) { /* platform-specific code can initalize it in theory */
-		atari_screen = (ULONG *) Util_malloc(ATARI_HEIGHT * ATARI_WIDTH);
+	if (Screen_atari == NULL) { /* platform-specific code can initialize it in theory */
+		Screen_atari = (ULONG *) Util_malloc(Screen_HEIGHT * Screen_WIDTH);
 #ifdef DIRTYRECT
-		screen_dirty = (UBYTE *) Util_malloc(ATARI_HEIGHT * ATARI_WIDTH / 8);
-		entire_screen_dirty();
+		Screen_dirty = (UBYTE *) Util_malloc(Screen_HEIGHT * Screen_WIDTH / 8);
+		Screen_EntireDirty();
 #endif
 #ifdef BITPL_SCR
-		atari_screen_b = (ULONG *) Util_malloc(ATARI_HEIGHT * ATARI_WIDTH);
-		atari_screen1 = atari_screen;
-		atari_screen2 = atari_screen_b;
+		Screen_atari_b = (ULONG *) Util_malloc(Screen_HEIGHT * Screen_WIDTH);
+		Screen_atari1 = Screen_atari;
+		Screen_atari2 = Screen_atari_b;
 #endif
 	}
+
+	return TRUE;
 }
 
 #define SMALLFONT_WIDTH    5
@@ -167,7 +183,7 @@ void Screen_Initialise(int *argc, char *argv[])
 
 static void SmallFont_DrawChar(UBYTE *screen, int ch, UBYTE color1, UBYTE color2)
 {
-	static const UBYTE font[11][SMALLFONT_HEIGHT] = {
+	static const UBYTE font[12][SMALLFONT_HEIGHT] = {
 		{
 			SMALLFONT_____,
 			SMALLFONT__X__,
@@ -266,6 +282,15 @@ static void SmallFont_DrawChar(UBYTE *screen, int ch, UBYTE color1, UBYTE color2
 			SMALLFONT_X___,
 			SMALLFONT_X_X_,
 			SMALLFONT_____
+		},
+		{
+			SMALLFONT_____,
+			SMALLFONT__X__,
+			SMALLFONT_X_X_,
+			SMALLFONT_X___,
+			SMALLFONT_X_X_,
+			SMALLFONT__X__,
+			SMALLFONT_____
 		}
 	};
 	int y;
@@ -274,10 +299,10 @@ static void SmallFont_DrawChar(UBYTE *screen, int ch, UBYTE color1, UBYTE color2
 		int mask;
 		src = font[ch][y];
 		for (mask = 1 << (SMALLFONT_WIDTH - 1); mask != 0; mask >>= 1) {
-			video_putbyte(screen, (UBYTE) ((src & mask) != 0 ? color1 : color2));
+			ANTIC_VideoPutByte(screen, (UBYTE) ((src & mask) != 0 ? color1 : color2));
 			screen++;
 		}
-		screen += ATARI_WIDTH - SMALLFONT_WIDTH;
+		screen += Screen_WIDTH - SMALLFONT_WIDTH;
 	}
 }
 
@@ -290,20 +315,22 @@ static void SmallFont_DrawInt(UBYTE *screen, int n, UBYTE color1, UBYTE color2)
 	} while (n > 0);
 }
 
-void Screen_DrawAtariSpeed(void)
+void Screen_DrawAtariSpeed(double cur_time)
 {
-	if (show_atari_speed) {
+	if (Screen_show_atari_speed) {
 		static int percent_display = 100;
 		static int last_updated = 0;
-		if ((nframes - last_updated) >= 25) {
-			last_updated = nframes;
-			percent_display = percent_atari_speed;
+		static double last_time = 0;
+		if ((cur_time - last_time) >= 0.5) {
+			percent_display = (int) (100 * (Atari800_nframes - last_updated) / (cur_time - last_time) / (Atari800_tv_mode == Atari800_TV_PAL ? 50 : 60));
+			last_updated = Atari800_nframes;
+			last_time = cur_time;
 		}
-		/* don't show if 99-101% */
-		if (percent_display < 99 || percent_display > 101) {
+		/* if (percent_display < 99 || percent_display > 101) */
+		{
 			/* space for 5 digits - up to 99999% Atari speed */
-			UBYTE *screen = (UBYTE *) atari_screen + screen_visible_x1 + 5 * SMALLFONT_WIDTH
-				          + (screen_visible_y2 - SMALLFONT_HEIGHT) * ATARI_WIDTH;
+			UBYTE *screen = (UBYTE *) Screen_atari + Screen_visible_x1 + 5 * SMALLFONT_WIDTH
+			          	+ (Screen_visible_y2 - SMALLFONT_HEIGHT) * Screen_WIDTH;
 			SmallFont_DrawChar(screen, SMALLFONT_PERCENT, 0x0c, 0x00);
 			SmallFont_DrawInt(screen - SMALLFONT_WIDTH, percent_display, 0x0c, 0x00);
 		}
@@ -312,15 +339,23 @@ void Screen_DrawAtariSpeed(void)
 
 void Screen_DrawDiskLED(void)
 {
-	if (sio_last_op_time > 0) {
+	if (SIO_last_op_time > 0) {
 		UBYTE *screen;
-		sio_last_op_time--;
-		screen = (UBYTE *) atari_screen + screen_visible_x2 - SMALLFONT_WIDTH
-			+ (screen_visible_y2 - SMALLFONT_HEIGHT) * ATARI_WIDTH;
-		if (show_disk_led)
-			SmallFont_DrawChar(screen, sio_last_drive, 0x00, (UBYTE) (sio_last_op == SIO_LAST_READ ? 0xac : 0x2b));
-		if (show_sector_counter)
-			SmallFont_DrawInt(screen - SMALLFONT_WIDTH, sio_last_sector, 0x00, 0x88);
+		if (SIO_last_drive != 0x60)
+			SIO_last_op_time--;
+		screen = (UBYTE *) Screen_atari + Screen_visible_x2 - SMALLFONT_WIDTH
+			+ (Screen_visible_y2 - SMALLFONT_HEIGHT) * Screen_WIDTH;
+		if (SIO_last_drive == 0x60 || SIO_last_drive == 0x61) {
+			if (Screen_show_disk_led)
+				SmallFont_DrawChar(screen, 11, 0x00, (UBYTE) (SIO_last_op == SIO_LAST_READ ? 0xac : 0x2b));
+		}
+		else {
+			if (Screen_show_disk_led)
+				SmallFont_DrawChar(screen, SIO_last_drive, 0x00, (UBYTE) (SIO_last_op == SIO_LAST_READ ? 0xac : 0x2b));
+		
+			if (Screen_show_sector_counter)
+				SmallFont_DrawInt(screen - SMALLFONT_WIDTH, SIO_last_sector, 0x00, 0x88);
+		}
 	}
 }
 
@@ -364,7 +399,7 @@ static void Screen_SavePCX(FILE *fp, UBYTE *ptr1, UBYTE *ptr2)
 	fputw(0, fp);     /* XMin */
 	fputw(0, fp);     /* YMin */
 	fputw(ATARI_VISIBLE_WIDTH - 1, fp); /* XMax */
-	fputw(ATARI_HEIGHT - 1, fp);        /* YMax */
+	fputw(Screen_HEIGHT - 1, fp);        /* YMax */
 	fputw(0, fp);     /* HRes */
 	fputw(0, fp);     /* VRes */
 	for (i = 0; i < 48; i++)
@@ -374,14 +409,14 @@ static void Screen_SavePCX(FILE *fp, UBYTE *ptr1, UBYTE *ptr2)
 	fputw(ATARI_VISIBLE_WIDTH, fp);  /* number of bytes per scan line per color plane */
 	fputw(1, fp);     /* palette info */
 	fputw(ATARI_VISIBLE_WIDTH, fp); /* screen resolution */
-	fputw(ATARI_HEIGHT, fp);
+	fputw(Screen_HEIGHT, fp);
 	for (i = 0; i < 54; i++)
 		fputc(0, fp);  /* unused */
 
-	for (y = 0; y < ATARI_HEIGHT; ) {
+	for (y = 0; y < Screen_HEIGHT; ) {
 		x = 0;
 		do {
-			last = ptr2 != NULL ? (((colortable[*ptr1] >> plane) & 0xff) + ((colortable[*ptr2] >> plane) & 0xff)) >> 1 : *ptr1;
+			last = ptr2 != NULL ? (((Colours_table[*ptr1] >> plane) & 0xff) + ((Colours_table[*ptr2] >> plane) & 0xff)) >> 1 : *ptr1;
 			count = 0xc0;
 			do {
 				ptr1++;
@@ -389,7 +424,7 @@ static void Screen_SavePCX(FILE *fp, UBYTE *ptr1, UBYTE *ptr2)
 					ptr2++;
 				count++;
 				x++;
-			} while (last == (ptr2 != NULL ? (((colortable[*ptr1] >> plane) & 0xff) + ((colortable[*ptr2] >> plane) & 0xff)) >> 1 : *ptr1)
+			} while (last == (ptr2 != NULL ? (((Colours_table[*ptr1] >> plane) & 0xff) + ((Colours_table[*ptr2] >> plane) & 0xff)) >> 1 : *ptr1)
 						&& count < 0xff && x < ATARI_VISIBLE_WIDTH);
 			if (count > 0xc1 || last >= 0xc0)
 				fputc(count, fp);
@@ -402,9 +437,9 @@ static void Screen_SavePCX(FILE *fp, UBYTE *ptr1, UBYTE *ptr2)
 			plane -= 8;
 		}
 		else {
-			ptr1 += ATARI_WIDTH - ATARI_VISIBLE_WIDTH;
+			ptr1 += Screen_WIDTH - ATARI_VISIBLE_WIDTH;
 			if (ptr2 != NULL) {
-				ptr2 += ATARI_WIDTH - ATARI_VISIBLE_WIDTH;
+				ptr2 += Screen_WIDTH - ATARI_VISIBLE_WIDTH;
 				plane = 16;
 			}
 			y++;
@@ -415,9 +450,9 @@ static void Screen_SavePCX(FILE *fp, UBYTE *ptr1, UBYTE *ptr2)
 		/* write palette */
 		fputc(0xc, fp);
 		for (i = 0; i < 256; i++) {
-			fputc(Palette_GetR(i), fp);
-			fputc(Palette_GetG(i), fp);
-			fputc(Palette_GetB(i), fp);
+			fputc(Colours_GetR(i), fp);
+			fputc(Colours_GetG(i), fp);
+			fputc(Colours_GetB(i), fp);
 		}
 	}
 }
@@ -436,7 +471,7 @@ static void Screen_SavePNG(FILE *fp, UBYTE *ptr1, UBYTE *ptr2)
 {
 	png_structp png_ptr;
 	png_infop info_ptr;
-	png_bytep rows[ATARI_HEIGHT];
+	png_bytep rows[Screen_HEIGHT];
 
 	png_ptr = png_create_write_struct(
 		PNG_LIBPNG_VER_STRING,
@@ -449,7 +484,7 @@ static void Screen_SavePNG(FILE *fp, UBYTE *ptr1, UBYTE *ptr2)
 		return;
 	png_init_io(png_ptr, fp);
 	png_set_IHDR(
-		png_ptr, info_ptr, ATARI_VISIBLE_WIDTH, ATARI_HEIGHT,
+		png_ptr, info_ptr, ATARI_VISIBLE_WIDTH, Screen_HEIGHT,
 		8, ptr2 == NULL ? PNG_COLOR_TYPE_PALETTE : PNG_COLOR_TYPE_RGB,
 		PNG_INTERLACE_NONE,
 		PNG_COMPRESSION_TYPE_DEFAULT,
@@ -459,32 +494,32 @@ static void Screen_SavePNG(FILE *fp, UBYTE *ptr1, UBYTE *ptr2)
 		int i;
 		png_color palette[256];
 		for (i = 0; i < 256; i++) {
-			palette[i].red = Palette_GetR(i);
-			palette[i].green = Palette_GetG(i);
-			palette[i].blue = Palette_GetB(i);
+			palette[i].red = Colours_GetR(i);
+			palette[i].green = Colours_GetG(i);
+			palette[i].blue = Colours_GetB(i);
 		}
 		png_set_PLTE(png_ptr, info_ptr, palette, 256);
-		for (i = 0; i < ATARI_HEIGHT; i++) {
+		for (i = 0; i < Screen_HEIGHT; i++) {
 			rows[i] = ptr1;
-			ptr1 += ATARI_WIDTH;
+			ptr1 += Screen_WIDTH;
 		}
 	}
 	else {
 		png_bytep ptr3;
 		int x;
 		int y;
-		ptr3 = (png_bytep) Util_malloc(3 * ATARI_VISIBLE_WIDTH * ATARI_HEIGHT);
-		for (y = 0; y < ATARI_HEIGHT; y++) {
+		ptr3 = (png_bytep) Util_malloc(3 * ATARI_VISIBLE_WIDTH * Screen_HEIGHT);
+		for (y = 0; y < Screen_HEIGHT; y++) {
 			rows[y] = ptr3;
 			for (x = 0; x < ATARI_VISIBLE_WIDTH; x++) {
-				*ptr3++ = (png_byte) ((Palette_GetR(*ptr1) + Palette_GetR(*ptr2)) >> 1);
-				*ptr3++ = (png_byte) ((Palette_GetG(*ptr1) + Palette_GetG(*ptr2)) >> 1);
-				*ptr3++ = (png_byte) ((Palette_GetB(*ptr1) + Palette_GetB(*ptr2)) >> 1);
+				*ptr3++ = (png_byte) ((Colours_GetR(*ptr1) + Colours_GetR(*ptr2)) >> 1);
+				*ptr3++ = (png_byte) ((Colours_GetG(*ptr1) + Colours_GetG(*ptr2)) >> 1);
+				*ptr3++ = (png_byte) ((Colours_GetB(*ptr1) + Colours_GetB(*ptr2)) >> 1);
 				ptr1++;
 				ptr2++;
 			}
-			ptr1 += ATARI_WIDTH - ATARI_VISIBLE_WIDTH;
-			ptr2 += ATARI_WIDTH - ATARI_VISIBLE_WIDTH;
+			ptr1 += Screen_WIDTH - ATARI_VISIBLE_WIDTH;
+			ptr2 += Screen_WIDTH - ATARI_VISIBLE_WIDTH;
 		}
 	}
 	png_set_rows(png_ptr, info_ptr, rows);
@@ -499,7 +534,7 @@ int Screen_SaveScreenshot(const char *filename, int interlaced)
 {
 	int is_png;
 	FILE *fp;
-	ULONG *main_atari_screen;
+	ULONG *main_screen_atari;
 	UBYTE *ptr1;
 	UBYTE *ptr2;
 	if (striendswith(filename, ".pcx"))
@@ -513,12 +548,12 @@ int Screen_SaveScreenshot(const char *filename, int interlaced)
 	fp = fopen(filename, "wb");
 	if (fp == NULL)
 		return FALSE;
-	main_atari_screen = atari_screen;
-	ptr1 = (UBYTE *) atari_screen + ATARI_LEFT_MARGIN;
+	main_screen_atari = Screen_atari;
+	ptr1 = (UBYTE *) Screen_atari + ATARI_LEFT_MARGIN;
 	if (interlaced) {
-		atari_screen = (ULONG *) Util_malloc(ATARI_WIDTH * ATARI_HEIGHT);
-		ptr2 = (UBYTE *) atari_screen + ATARI_LEFT_MARGIN;
-		ANTIC_Frame(TRUE); /* draw on atari_screen */
+		Screen_atari = (ULONG *) Util_malloc(Screen_WIDTH * Screen_HEIGHT);
+		ptr2 = (UBYTE *) Screen_atari + ATARI_LEFT_MARGIN;
+		ANTIC_Frame(TRUE); /* draw on Screen_atari */
 	}
 	else {
 		ptr2 = NULL;
@@ -531,8 +566,8 @@ int Screen_SaveScreenshot(const char *filename, int interlaced)
 		Screen_SavePCX(fp, ptr1, ptr2);
 	fclose(fp);
 	if (interlaced) {
-		free(atari_screen);
-		atari_screen = main_atari_screen;
+		free(Screen_atari);
+		Screen_atari = main_screen_atari;
 	}
 	return TRUE;
 }
@@ -543,3 +578,10 @@ void Screen_SaveNextScreenshot(int interlaced)
 	Screen_FindScreenshotFilename(filename);
 	Screen_SaveScreenshot(filename, interlaced);
 }
+
+void Screen_EntireDirty(void)
+{
+#ifdef DIRTYRECT
+	memset(Screen_dirty, 1, Screen_WIDTH * Screen_HEIGHT / 8);
+#endif /* DIRTYRECT */
+}
