@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with Atari800; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include "config.h"
@@ -32,10 +32,12 @@
 #include "atari.h"
 #include "colours.h"
 #include "log.h"
+#include "screen.h"
 #include "util.h"
 
 #include "main.h"
 #include "screen_win32.h"
+#include "platform.h"
 
 #define SHOWFRAME 0
 
@@ -76,7 +78,7 @@ void groff(void)
 	}
 }
 
-LRESULT CALLBACK Atari_WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
+/* LRESULT CALLBACK Atari_WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam); */  /* in main.c */
 
 static BOOL initwin(void)
 {
@@ -115,7 +117,7 @@ static BOOL initwin(void)
 static int initFail(HWND hwnd, const char *func, HRESULT hr)
 {
 	char txt[256];
-	sprintf(txt, "DirectDraw Init FAILED: %s returned 0x%x", func, hr);
+	sprintf(txt, "DirectDraw Init FAILED: %s returned 0x%x", func, (unsigned int)hr);
 	MessageBox(hwnd, txt, myname, MB_OK);
 	groff();
 	DestroyWindow(hwnd);
@@ -140,9 +142,9 @@ int gron(int *argc, char *argv[])
 			bltgfx = TRUE;
 		else {
 			if (strcmp(argv[i], "-help") == 0) {
-				Aprint("\t-windowed        Run in a window");
-				Aprint("\t-width <num>     Set display mode width");
-				Aprint("\t-blt             Use blitting to draw graphics");
+				Log_print("\t-windowed        Run in a window");
+				Log_print("\t-width <num>     Set display mode width");
+				Log_print("\t-blt             Use blitting to draw graphics");
 				help = TRUE;
 			}
 			argv[j++] = argv[i];
@@ -233,7 +235,7 @@ int gron(int *argc, char *argv[])
 	}
 	else {
 		for (i = 0; i < MAX_CLR; i++)
-			palette(i, Palette_GetR(i), Palette_GetG(i), Palette_GetB(i));
+			palette(i, Colours_GetR(i), Colours_GetG(i), Colours_GetB(i));
 		IDirectDraw4_CreatePalette(lpDD, DDPCAPS_8BIT, pal, &lpDDPal, NULL);
 		if (lpDDPal)
 			IDirectDrawSurface4_SetPalette(lpDDSPrimary, lpDDPal);
@@ -257,14 +259,26 @@ void palette(int ent, UBYTE r, UBYTE g, UBYTE b)
 	pal[ent].peFlags = 0;
 }
 
-void refreshv_win32api(UBYTE *scr_ptr)
+/* Platform-specific function to update the palette if it changed */
+void PLATFORM_PaletteUpdate(void)
+{
+	if(lpDDPal != NULL) {
+		int i;
+		for (i = 0; i < MAX_CLR; i++) {
+			palette(i, Colours_GetR(i), Colours_GetG(i), Colours_GetB(i));
+		}
+		palupd(0, MAX_CLR);
+	}
+}
+
+static void refreshv_win32api(UBYTE *scr_ptr)
 {
 	PAINTSTRUCT ps;
 	HDC hdc;
 	HDC hCdc;
 	BITMAPINFO bi;
 	DWORD *bitmap_bits = NULL;
-	ULONG   ulWindowWidth, ulWindowHeight;      // window width/height
+	ULONG   ulWindowWidth, ulWindowHeight;      /* window width/height */
 	HBITMAP hBitmap;
 	RECT rt;
 	int i;
@@ -276,42 +290,42 @@ void refreshv_win32api(UBYTE *scr_ptr)
 	ulWindowWidth = rt.right - rt.left;
 	ulWindowHeight = rt.bottom - rt.top;
 
-	// make sure we have at least some window size
+	/* make sure we have at least some window size */
 	if ((!ulWindowWidth) || (!ulWindowHeight))
 		return;
 
-	bi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER); // structure size in bytes
-	bi.bmiHeader.biWidth = ATARI_WIDTH-48;
-	bi.bmiHeader.biHeight = ATARI_HEIGHT;
+	bi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER); /* structure size in bytes */
+	bi.bmiHeader.biWidth = Screen_WIDTH-48;
+	bi.bmiHeader.biHeight = Screen_HEIGHT;
 	bi.bmiHeader.biPlanes = 1;
 	bi.bmiHeader.biBitCount = 32;
-	bi.bmiHeader.biCompression = BI_RGB; //BI_BITFIELDS OR BI_RGB???
-	bi.bmiHeader.biSizeImage = 0; // for BI_RGB set to 0
-	bi.bmiHeader.biXPelsPerMeter = 2952; //75dpi=2952bpm
-	bi.bmiHeader.biYPelsPerMeter = 2952; //75dpi=2952bpm
+	bi.bmiHeader.biCompression = BI_RGB; /* BI_BITFIELDS OR BI_RGB??? */
+	bi.bmiHeader.biSizeImage = 0; /* for BI_RGB set to 0 */
+	bi.bmiHeader.biXPelsPerMeter = 2952; /* 75dpi=2952bpm */
+	bi.bmiHeader.biYPelsPerMeter = 2952; /* 75dpi=2952bpm */
 	bi.bmiHeader.biClrUsed = 0;
 	bi.bmiHeader.biClrImportant = 0;
 
 	hCdc = CreateCompatibleDC(hdc);
-	hBitmap = CreateDIBSection(hCdc, &bi, DIB_RGB_COLORS, &bitmap_bits, NULL, 0);
+	hBitmap = CreateDIBSection(hCdc, &bi, DIB_RGB_COLORS, (void *)&bitmap_bits, NULL, 0);
 	if (!hBitmap) {
 		MessageBox(hWndMain, "Could not create bitmap", myname, MB_OK);
 		DestroyWindow(hWndMain);
 		return;
 	}
 
-	// Copying the atari screen to bitmap.
-	for (i = 0; i < ATARI_HEIGHT; i++) {
-		for (j = 0; j < ATARI_WIDTH - 48; j++) {
-			*bitmap_bits++ = colortable[*scr_ptr++];
+	/* Copying the atari screen to bitmap. */
+	for (i = 0; i < Screen_HEIGHT; i++) {
+		for (j = 0; j < Screen_WIDTH - 48; j++) {
+			*bitmap_bits++ = Colours_table[*scr_ptr++];
 		}
-		// The last 48 columns of the screen are not used
+		/* The last 48 columns of the screen are not used */
 		scr_ptr+=48;
 	}
 
 	SelectObject(hCdc, hBitmap);
 
-	// Draw the bitmap on the screen. The image must be felliped vertically
+	/* Draw the bitmap on the screen. The image must be flipped vertically */
 	if (!StretchBlt(hdc, 0, 0, ulWindowWidth, ulWindowHeight,
 			hCdc, 0, bi.bmiHeader.biHeight, bi.bmiHeader.biWidth, -bi.bmiHeader.biHeight, SRCCOPY)) {
 		MessageBox(hWndMain, "Could not StretchBlt", myname, MB_OK);
@@ -331,7 +345,8 @@ void refreshv(UBYTE *scr_ptr)
 	DDSURFACEDESC2 desc0;
 	int err;
 	int x, y;
-	UBYTE *src;
+	UBYTE *srcb;
+	ULONG *srcl;
 	ULONG *dst;
 	int h, w;
 	DDBLTFX ddbltfx;
@@ -351,27 +366,27 @@ void refreshv(UBYTE *scr_ptr)
 		scraddr = (UBYTE *) desc0.lpSurface + (bltgfx ? linesize * 6 : 0);
 
 		if (bltgfx) {
-			for (y = 0; y < ATARI_HEIGHT; y++) {
+			for (y = 0; y < Screen_HEIGHT; y++) {
 				dst = (ULONG *) (scraddr + y * linesize);
-				src = scr_ptr + y * ATARI_WIDTH;
+				srcb = scr_ptr + y * Screen_WIDTH;
 				for (x = 0; x < scrwidth; x++)
-					*dst++ = colortable[*src++];
+					*dst++ = Colours_table[*srcb++];
 			}
 		}
 		else {
 			w = (scrwidth - 336) / 2;
-			h = (scrheight - ATARI_HEIGHT) / 2;
+			h = (scrheight - Screen_HEIGHT) / 2;
 			if (w > 0)
 				scraddr += w;
 			else if (w < 0)
 				scr_ptr -= w;
 			if (h > 0)
 				scraddr += linesize * h;
-			for (y = 0; y < ATARI_HEIGHT; y++) {
+			for (y = 0; y < Screen_HEIGHT; y++) {
 				dst = (ULONG *) (scraddr + y * linesize);
-				src = scr_ptr + y * ATARI_WIDTH;
+				srcl = (ULONG *) (scr_ptr + y * Screen_WIDTH);
 				for (x = (w >= 0) ? (336 >> 2) : (scrwidth >> 2); x > 0; x--)
-					*dst++ = *((ULONG *) src)++;
+					*dst++ = *srcl++;
 			}
 		}
 
