@@ -1,5 +1,5 @@
-#ifndef _SCREEN_WIN32_H_
-#define _SCREEN_WIN32_H_
+#ifndef SCREEN_WIN32_H_
+#define SCREEN_WIN32_H_
 
 #include "atari.h"
 
@@ -12,4 +12,4 @@ void palette(int ent, UBYTE r, UBYTE g, UBYTE b);
 void refreshv(UBYTE * scr_ptr);
 extern int windowed;
 
-#endif /* _SCREEN_H_ */
+#endif /* SCREEN_WIN32_H_ */
