@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with Atari800; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 /* Based on Win32 port by  Krzysztof Nikiel */
@@ -45,7 +45,8 @@ static int kbjoy = 1;
 
 DWORD REG_bat, REG_ac, REG_disp, bat_timeout;
 
-void Aprint(char *format, ... )
+/* XXX: don't define Log_print, use a platform-specific output in log.c */
+void Log_print(char *format, ... )
 {
 #ifdef DEBUG
 	char tmp1[512];
@@ -59,7 +60,7 @@ void Aprint(char *format, ... )
 	OutputDebugString(tmp2);
 #endif
 }
-void Aflushlog(void) {}
+void Log_flushlog(void) {}
 
 static DWORD reg_access(TCHAR *key, TCHAR *val, DWORD data)
 {
@@ -103,7 +104,7 @@ static void backlight_xchg(void)
 	}
 }
 
-int Atari_Keyboard(void)
+int PLATFORM_Keyboard(void)
 {
 	int keycode;
 
@@ -113,7 +114,7 @@ int Atari_Keyboard(void)
 	return keycode;
 }
 
-void Atari_Initialise(int *argc, char *argv[])
+int PLATFORM_Initialise(int *argc, char *argv[])
 {
 #ifdef SOUND
 	Sound_Initialise(argc, argv);
@@ -137,9 +138,11 @@ void Atari_Initialise(int *argc, char *argv[])
 	SystemParametersInfo(SPI_SETBATTERYIDLETIMEOUT, 60 * 60 * 2, NULL, SPIF_SENDCHANGE);
 
 	clearkb();
+
+	return TRUE;
 }
 
-int Atari_Exit(int run_monitor)
+int PLATFORM_Exit(int run_monitor)
 {
 	backlight_xchg();	/* restore backlight settings */
 	SystemParametersInfo(SPI_SETBATTERYIDLETIMEOUT, bat_timeout, NULL, SPIF_SENDCHANGE);
@@ -149,7 +152,7 @@ int Atari_Exit(int run_monitor)
 		return 1;
 
 #ifdef BUFFERED_LOG
-	Aflushlog();
+	Log_flushlog();
 #endif
 
 	uninitinput();
@@ -162,12 +165,12 @@ int Atari_Exit(int run_monitor)
 	return 0;
 }
 
-void Atari_DisplayScreen(void)
+void PLATFORM_DisplayScreen(void)
 {
-	refreshv((UBYTE *) atari_screen + 24);
+	refreshv((UBYTE *) Screen_atari + 24);
 }
 
-int Atari_PORT(int num)
+int PLATFORM_PORT(int num)
 {
 	if (num == 0)
 		return stick0;
@@ -176,7 +179,7 @@ int Atari_PORT(int num)
 }
 
 
-int Atari_TRIG(int num)
+int PLATFORM_TRIG(int num)
 {
 	if (num == 0)
 		return trig0;
@@ -184,19 +187,19 @@ int Atari_TRIG(int num)
 		return 1;
 }
 
-void Atari_ConfigInit(void)
+void PLATFORM_ConfigInit(void)
 {
 	enable_new_pokey = 0;
-	screen_visible_x1 = 24;
-	screen_visible_y1 = 10;
-	screen_visible_x2 = 344;
-	screen_visible_y2 = 230;
-	show_disk_led = 1;
-	show_sector_counter = 1;
-	show_atari_speed = 1;
+	Screen_visible_x1 = 24;
+	Screen_visible_y1 = 10;
+	Screen_visible_x2 = 344;
+	Screen_visible_y2 = 230;
+	Screen_show_disk_led = 1;
+	Screen_show_sector_counter = 1;
+	Screen_show_atari_speed = 1;
 }
 
-int Atari_Configure(char* option, char *parameters)
+int PLATFORM_Configure(char* option, char *parameters)
 {
 	if (strcmp(option, "WCE_LINEAR_FILTER") == 0)
 	{
@@ -217,7 +220,7 @@ int Atari_Configure(char* option, char *parameters)
 	return 0;
 }
 
-void Atari_ConfigSave(FILE *fp)
+void PLATFORM_ConfigSave(FILE *fp)
 {
 	fprintf(fp, "WCE_LINEAR_FILTER=%d\n", smooth_filter);
 	fprintf(fp, "WCE_VIRTUAL_JOYSTICK=%d\n", virtual_joystick);
@@ -241,7 +244,7 @@ void AboutPocketAtari(void)
 		"\0"
 		"\0"
 		"Atari core for this version\0"
-		ATARI_TITLE "\0"
+		Atari800_TITLE "\0"
 		"http://atari800.sf.net\0"
 		"\n"
 	:
@@ -251,7 +254,7 @@ void AboutPocketAtari(void)
 		"\0"
 		"\0"
 		"This port is based on\0"
-		ATARI_TITLE "\0"
+		Atari800_TITLE "\0"
 		"http://atari800.sf.net\0"
 		"\0"
 		"PocketPC port update and\0"
@@ -274,10 +277,10 @@ int wince_main(int argc, char **argv)
 	{
 		if (emulator_active)
 		{
-			key_code = Atari_Keyboard();
+			INPUT_key_code = PLATFORM_Keyboard();
 			Atari800_Frame();
-			if (display_screen)
-				Atari_DisplayScreen();
+			if (Atari800_display_screen)
+				PLATFORM_DisplayScreen();
 		}
 		else
 		{
