@@ -13,7 +13,7 @@
 #define SUPPORTS_ATARI_CONFIGINIT		1
 #undef  WIN32
 #define WIN32							1
-#define KB_UI							1
+#define USE_UI_BASIC_ONSCREEN_KEYBOARD				1
 
 #define perror(s) wce_perror(s)
 void wce_perror(char *s);
